

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class ApiKey:
    """
    Represents api key information
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    The api key id
    """
    @property
    def api_key_id(self) -> Uuid:
        return Uuid(self.dictionary.get("apiKeyId"))
    """
    The name of the api key
    """
    @property
    def name(self) -> String:
        return String(self.dictionary.get("name"))
    """
    The description of the api key
    """
    @property
    def description(self) -> String:
        return String(self.dictionary.get("description"))
    """
    The flag to enable the api key
    """
    @property
    def enabled(self) -> Boolean:
        return Boolean(self.dictionary.get("enabled"))



class RemoveFileImpulse:
    uri: str

    def __init__(self, uri: str):
        self.uri = uri

    def __eq__(self, other):
        return self.uri == other.uri

    def __repr__(self):
        return {'uri': self.uri}



class FileListing:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        return self.dictionary == other.dictionary

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def tenant(self):
        return self.dictionary.get("tenant")

    @property
    def file_path(self):
        return self.dictionary.get("filePath")

    @property
    def last_modified(self):
        return self.dictionary.get("lastModified")

    @property
    def size(self):
        return self.dictionary.get("size")




from gaia_sdk.graphql.request.type.BehaviourNodeExecution import BehaviourNodeExecution

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class BehaviourExecutionDetail(list):
    """
    Represents a detailed summary of executed entities to a given processInstanceId
    """

    def process_instance_id(self):
        self.append(lambda x: "processInstanceId")

    def identity_id(self):
        self.append(lambda x: "identityId")

    def qualifier(self):
        self.append(lambda x: "qualifier")

    def behaviours(self):
        self.append(lambda x: "behaviours")

    def behaviour_id(self):
        self.append(lambda x: "behaviourId")

    def nodes(self, config: Callable[['BehaviourNodeExecution'], None]):
        def callback(registry: VariableRegistry):
            entity = BehaviourNodeExecution()
            config(entity)
            return "nodes {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class DeleteUserImpulse():
    """
    The specification to delete a user instance
    """
    userId: str

    def __init__(self, userId: str):
        self.userId = userId

    def __eq__(self, other):
        if type(other) is type(self):
            return self.userId == other.userId
        return False

    def __repr__(self):
        return {'userId': self.userId}



import time
import unittest
from uuid import uuid4

from rx import operators as ops
from rx import pipe

from gaia_sdk.tests.mock import mock_gaia_ref

class TestConcurrency(unittest.TestCase):

    def test_does_not_block(self):
        gaia_ref = mock_gaia_ref(self.return_after_seconds(5))

        def config(x):
            x.identity_id()
            x.qualifier()

        t0 = time.perf_counter()
        pipe(ops.first())(gaia_ref.retrieve_intents(str(uuid4()), config))
        t1 = time.perf_counter()
        exec_time = t1 - t0

        self.assertLess(exec_time, 5)

    def return_after_seconds(self, t):
        def fn(*args):
            time.sleep(t)
            return {'data': {'retrieve': {'knowledge': {'intents': [{
                'identityId': "0",
                'reference': 'abc',
                'qualifier': 'some_intent'

            }]}}}, 'errors': {}}

        return fn





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class ConnectSetNodeImpulse():
    """
    The specification to set an edge for a node
    """
    type: EdgeType
    target: str
    properties: dict
    weight: float

    def __init__(self, type: EdgeType, target: str, properties: dict, weight: float):
        self.type = type
        self.target = target
        self.properties = properties
        self.weight = weight

    def __eq__(self, other):
        if type(other) is type(self):
            return self.type == other.type and self.target == other.target and self.properties == other.properties and self.weight == other.weight
        return False

    def __repr__(self):
        return {'type': self.type, 'target': self.target, 'properties': self.properties, 'weight': self.weight}




from gaia_sdk.graphql.response.type.Evaluation import Evaluation
from gaia_sdk.graphql.response.type.Preservation import Preservation
from gaia_sdk.graphql.response.type.Practice import Practice
from gaia_sdk.graphql.response.type.Perception import Perception
from gaia_sdk.graphql.response.type.Activation import Activation

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Mutation:
    """
    The top level mutation type
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    Sensor impulses for all perception based functions.
        Perceptions are used to invoke events within gaia.
    """
    @property
    def perceive(self) -> Perception:
        return Perception(self.dictionary.get("perceive"))
    """
    Sensor impulses for all practice based functions.
        Practices are used to transfer skills to gaia and to train them.
    """
    @property
    def practice(self) -> Practice:
        return Practice(self.dictionary.get("practice"))
    """
    Sensor impulses for all preservation based functions.
        Preservations are used to invoke create/update/delete functions.
    """
    @property
    def preserve(self) -> Preservation:
        return Preservation(self.dictionary.get("preserve"))
    """
    Container element for all evaluate sensor fields.
        Evaluations are used to invoke skills and to return the result.
    """
    @property
    def evaluate(self) -> Evaluation:
        return Evaluation(self.dictionary.get("evaluate"))
    """
    Container element for all evaluate sensor fields.
        The activation can be used to unseal the vault or to grant access to an user.
    """
    @property
    def activate(self) -> Activation:
        return Activation(self.dictionary.get("activate"))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Identity(list):
    """
    Represents identity information
    """

    """
    The identity id
    """
    def identity_id(self):
        self.append(lambda x: "identityId")

    """
    The tenant id
    """
    def tenant_id(self):
        self.append(lambda x: "tenantId")

    """
    The name of the identity
    """
    def qualifier(self):
        self.append(lambda x: "qualifier")

    """
    The available languages of the identity
    """
    def available_languages(self):
        self.append(lambda x: "availableLanguages")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.response.type.PerceivedImpulse import PerceivedImpulse
from gaia_sdk.graphql.request.input.PerceiveReceptionImpulse import PerceiveReceptionImpulse
from gaia_sdk.graphql.request.input.PerceiveSuggestionImpulse import PerceiveSuggestionImpulse
from gaia_sdk.graphql.request.input.PerceiveButtonImpulse import PerceiveButtonImpulse
from gaia_sdk.graphql.request.input.PerceiveUtteranceImpulse import PerceiveUtteranceImpulse
from gaia_sdk.graphql.request.input.PerceiveSubmitImpulse import PerceiveSubmitImpulse

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Conversational:
    """
    Type which contains all impulses needed for the maintainence of a conversation
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    Utterance perception impulse used to send an utterance text to gaia
    """
    @property
    def perceive_utterance(self) -> PerceivedImpulse:
        return PerceivedImpulse(self.dictionary.get("perceiveUtterance"))
    """
    Button perception impulse used to send a button action to gaia
    """
    @property
    def perceive_button(self) -> PerceivedImpulse:
        return PerceivedImpulse(self.dictionary.get("perceiveButton"))
    """
    Submit perception impulse used to send a submit action to gaia
    """
    @property
    def perceive_submit(self) -> PerceivedImpulse:
        return PerceivedImpulse(self.dictionary.get("perceiveSubmit"))
    """
    Reception perception impulse used to send a reception to gaia
    """
    @property
    def perceive_reception(self) -> PerceivedImpulse:
        return PerceivedImpulse(self.dictionary.get("perceiveReception"))
    """
    Suggestion perception impulse used to send a suggestion action to gaia
    """
    @property
    def perceive_suggestion(self) -> PerceivedImpulse:
        return PerceivedImpulse(self.dictionary.get("perceiveSuggestion"))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class ConnectUnsetNodeImpulse():
    """
    The specification to unset edges of a certain type for a node
    """
    type: EdgeType

    def __init__(self, type: EdgeType):
        self.type = type

    def __eq__(self, other):
        if type(other) is type(self):
            return self.type == other.type
        return False

    def __repr__(self):
        return {'type': self.type}



import logging
import unittest
from typing import Callable, Union
from uuid import uuid4

from rx import operators as ops, pipe

from gaia_sdk.graphql.request.input.CreateBehaviourImpulse import CreateBehaviourImpulse
from gaia_sdk.graphql.request.input.CreateCodeImpulse import CreateCodeImpulse
from gaia_sdk.graphql.request.input.CreateEdgeImpulse import CreateEdgeImpulse
from gaia_sdk.graphql.request.input.CreateFulfilmentImpulse import CreateFulfilmentImpulse
from gaia_sdk.graphql.request.input.CreateIdentityImpulse import CreateIdentityImpulse
from gaia_sdk.graphql.request.input.CreateIntentImpulse import CreateIntentImpulse
from gaia_sdk.graphql.request.input.CreatePromptImpulse import CreatePromptImpulse
from gaia_sdk.graphql.request.input.CreateStatementImpulse import CreateStatementImpulse
from gaia_sdk.graphql.request.input.DeleteBehaviourImpulse import DeleteBehaviourImpulse
from gaia_sdk.graphql.request.input.DeleteCodeImpulse import DeleteCodeImpulse
from gaia_sdk.graphql.request.input.DeleteEdgeImpulse import DeleteEdgeImpulse
from gaia_sdk.graphql.request.input.DeleteFulfilmentImpulse import DeleteFulfilmentImpulse
from gaia_sdk.graphql.request.input.DeleteIdentityImpulse import DeleteIdentityImpulse
from gaia_sdk.graphql.request.input.DeleteIntentImpulse import DeleteIntentImpulse
from gaia_sdk.graphql.request.input.DeletePromptImpulse import DeletePromptImpulse
from gaia_sdk.graphql.request.input.DeleteStatementImpulse import DeleteStatementImpulse
from gaia_sdk.graphql.request.input.UpdateBehaviourImpulse import UpdateBehaviourImpulse
from gaia_sdk.graphql.request.input.UpdateCodeImpulse import UpdateCodeImpulse
from gaia_sdk.graphql.request.input.UpdateFulfilmentImpulse import UpdateFulfilmentImpulse
from gaia_sdk.graphql.request.input.UpdateIdentityImpulse import UpdateIdentityImpulse
from gaia_sdk.graphql.request.input.UpdateIntentImpulse import UpdateIntentImpulse
from gaia_sdk.graphql.request.input.UpdatePromptImpulse import UpdatePromptImpulse
from gaia_sdk.graphql.request.input.UpdateStatementImpulse import UpdateStatementImpulse
from gaia_sdk.graphql.request.type.DeletedCodeImpulse import DeletedCodeImpulse
from gaia_sdk.graphql.response.type.CreatedBehaviourImpulse import CreatedBehaviourImpulse
from gaia_sdk.graphql.response.type.CreatedCodeImpulse import CreatedCodeImpulse
from gaia_sdk.graphql.response.type.CreatedEdgeImpulse import CreatedEdgeImpulse
from gaia_sdk.graphql.response.type.CreatedFulfilmentImpulse import CreatedFulfilmentImpulse
from gaia_sdk.graphql.response.type.CreatedIdentityImpulse import CreatedIdentityImpulse
from gaia_sdk.graphql.response.type.CreatedIntentImpulse import CreatedIntentImpulse
from gaia_sdk.graphql.response.type.CreatedPromptImpulse import CreatedPromptImpulse
from gaia_sdk.graphql.response.type.CreatedStatementImpulse import CreatedStatementImpulse
from gaia_sdk.graphql.response.type.DeletedBehaviourImpulse import DeletedBehaviourImpulse
from gaia_sdk.graphql.response.type.DeletedEdgeImpulse import DeletedEdgeImpulse
from gaia_sdk.graphql.response.type.DeletedFulfilmentImpulse import DeletedFulfilmentImpulse
from gaia_sdk.graphql.response.type.DeletedIdentityImpulse import DeletedIdentityImpulse
from gaia_sdk.graphql.response.type.DeletedIntentImpulse import DeletedIntentImpulse
from gaia_sdk.graphql.response.type.DeletedPromptImpulse import DeletedPromptImpulse
from gaia_sdk.graphql.response.type.DeletedStatementImpulse import DeletedStatementImpulse
from gaia_sdk.graphql.response.type.UpdatedBehaviourImpulse import UpdatedBehaviourImpulse
from gaia_sdk.graphql.response.type.UpdatedCodeImpulse import UpdatedCodeImpulse
from gaia_sdk.graphql.response.type.UpdatedFulfilmentImpulse import UpdatedFulfilmentImpulse
from gaia_sdk.graphql.response.type.UpdatedIdentityImpulse import UpdatedIdentityImpulse
from gaia_sdk.graphql.response.type.UpdatedIntentImpulse import UpdatedIntentImpulse
from gaia_sdk.graphql.response.type.UpdatedPromptImpulse import UpdatedPromptImpulse
from gaia_sdk.graphql.response.type.UpdatedStatementImpulse import UpdatedStatementImpulse

from gaia_sdk.tests.mock import mock_gaia_ref
from gaia_sdk.tests.mock import MockResponse

logging.basicConfig(level=logging.DEBUG)


class RxException(Exception):
    pass


class TestPreservation(unittest.TestCase):

    ####################################################################################################################
    # Mock responses
    ####################################################################################################################
    def setUp(self) -> None:
        self.identityMockCreateUpdateResponse: Callable[[str], MockResponse] = lambda t: MockResponse(
            {"data": {"preserve": {t: {"identities": [
                {"id": "asdf", "data": {"identityId": "i1",
                                        "tenantId": "foo",
                                        "qualifier": "bar",
                                        "availableLanguages": {"de": "Deutsch"}}}
            ]}}}}
        )

        self.identityDeleteMockResponse: Callable[[str], MockResponse] = lambda: MockResponse(
            {"data": {"preserve": {"delete": {"identities": [
                {"id": "asdf", "data": {"identityId": "i1"}}
            ]}}}}
        )

        self.deleteMockResponse: Callable[[str], MockResponse] = lambda t: MockResponse(
            {"data": {"preserve": {"delete": {t: [
                {"id": "asdf", "data": {"identityId": "i1",
                                        "reference": "r1"}}
            ]}}}}
        )

        """Entity = ("prompt" | "intent" | "fulfilment" | "statement") """
        self.entityCreateUpdateMockResponse: Callable[[str], MockResponse] = lambda t1, t2: MockResponse(
            {"data": {"preserve": {t1: {t2: [
                {"id": "asdf", "data": {"identityId": "i1",
                                        "reference": "r1",
                                        "qualifier": "foo",
                                        "appendent": "bar",
                                        "utterance": {"de": ["content"]},
                                        "labelList": [],
                                        "version": "2.7.0-SNAPSHOT"}}
            ]}}}}
        )

        self.entityDeleteMockResponse: Callable[[str], MockResponse] = lambda t: self.deleteMockResponse(t)

        self.behaviourCreateUpdateMockResponse: Callable[[str], MockResponse] = lambda t: MockResponse(
            {"data": {"preserve": {t: {"behaviours": [
                {"id": "asdf", "data": {"identityId": "i1",
                                        "reference": "r1",
                                        "qualifier": "foo",
                                        "appendent": "bar",
                                        "labelList": []}}
            ]}}}}
        )

        self.behaviourDeleteMockResponse: Callable[[str], MockResponse] = lambda: self.deleteMockResponse("behaviours")

        self.codeCreateUpdateMockResponse: Callable[[str], MockResponse] = lambda t: MockResponse(
            {"data": {"preserve": {t: {"codes": [
                {"id": "asdf", "data": {"identityId": "i1",
                                        "reference": "r1",
                                        "qualifier": "foo",
                                        "appendent": "bar",
                                        "code": {"index": "httpGet(\"https://postman-echo.com/get?foo=bar\")"},
                                        "type": "atreus",
                                        "labelList": []}}
            ]}}}}
        )

        self.codeDeleteMockResponse: Callable[[str], MockResponse] = lambda: self.deleteMockResponse("codes")

        self.edgeCreateMockResponse: Callable[[str], MockResponse] = lambda: MockResponse(
            {"data": {"preserve": {"create": {"edges": [
                {"id": "asdf", "data": {"source": "n1",
                                        "target": "n2",
                                        "type": "someType",
                                        "weight": 2.7,
                                        "edgeId": "e1",
                                        "properties": {}}}
            ]}}}}
        )

        self.edgeDeleteMockResponse: Callable[[str], MockResponse] = lambda: MockResponse(
            {"data": {"preserve": {"delete": {"edges": [
                {"id": "asdf", "data": {"source": "n1",
                                        "edgeId": "e1"}}
            ]}}}}
        )

    ####################################################################################################################
    # Assertions
    ####################################################################################################################
    def assert_create_update_identity(self, result: Union[CreatedIdentityImpulse, UpdatedIdentityImpulse]):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("identityId"), "i1")
        self.assertEqual(data.get("tenantId"), "foo")
        self.assertEqual(data.get("qualifier"), "bar")
        self.assertEqual(data.get("availableLanguages"),  {"de": "Deutsch"})

    def assert_delete_identity(self, result: DeletedIdentityImpulse):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("identityId"), "i1")

    def assert_delete(self, result: Union[DeletedIntentImpulse,
                                          DeletedPromptImpulse,
                                          DeletedStatementImpulse,
                                          DeletedFulfilmentImpulse,
                                          DeletedBehaviourImpulse,
                                          DeletedCodeImpulse]):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("identityId"), "i1")
        self.assertEqual(data.get("reference"), "r1")

    def assert_create_update_entity(self, result: Union[CreatedIntentImpulse, UpdatedIntentImpulse,
                                                        CreatedPromptImpulse, UpdatedPromptImpulse,
                                                        CreatedStatementImpulse, UpdatedStatementImpulse,
                                                        CreatedFulfilmentImpulse, UpdatedFulfilmentImpulse]):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("identityId"), "i1")
        self.assertEqual(data.get("reference"), "r1")
        self.assertEqual(data.get("qualifier"), "foo")
        self.assertEqual(data.get("appendent"), "bar")
        self.assertEqual(data.get("utterance"), {"de": ["content"]})
        self.assertEqual(data.get("labelList"), [])
        self.assertEqual(data.get("version"),  "2.7.0-SNAPSHOT")

    def assert_delete_entity(self, result: Union[DeletedIntentImpulse,
                                                 DeletedPromptImpulse,
                                                 DeletedStatementImpulse,
                                                 DeletedFulfilmentImpulse]):
        self.assert_delete(result)

    def assert_create_update_behaviour(self, result: Union[CreatedBehaviourImpulse, UpdatedBehaviourImpulse]):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("identityId"), "i1")
        self.assertEqual(data.get("reference"), "r1")
        self.assertEqual(data.get("qualifier"), "foo")
        self.assertEqual(data.get("appendent"), "bar")
        self.assertEqual(data.get("labelList"), [])

    def assert_delete_behaviour(self, result: DeletedBehaviourImpulse):
        self.assert_delete(result)

    def assert_create_update_code(self, result: Union[CreatedCodeImpulse, UpdatedCodeImpulse]):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("identityId"), "i1")
        self.assertEqual(data.get("reference"), "r1")
        self.assertEqual(data.get("qualifier"), "foo")
        self.assertEqual(data.get("appendent"), "bar")
        self.assertEqual(data.get("code"), {"index": "httpGet(\"https://postman-echo.com/get?foo=bar\")"})
        self.assertEqual(data.get("type"), "atreus")
        self.assertEqual(data.get("labelList"), [])

    def assert_delete_code(self, result: DeletedCodeImpulse):
        self.assert_delete(result)

    def assert_create_edge(self, result: CreatedEdgeImpulse):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("source"), "n1")
        self.assertEqual(data.get("target"), "n2")
        self.assertEqual(data.get("type"), "someType")
        self.assertEqual(data.get("weight"), 2.7)
        self.assertEqual(data.get("edgeId"), "e1")
        self.assertEqual(data.get("properties"), {})

    def assert_delete_edge(self, result: DeletedEdgeImpulse):
        self.assertEqual(result.dictionary.get("id"), "asdf")
        data = result.dictionary.get("data")
        self.assertEqual(data.get("source"), "n1")
        self.assertEqual(data.get("edgeId"), "e1")

    ####################################################################################################################
    # Tests
    ####################################################################################################################
    def test_preserve_create_identity(self):
        gaia_ref = mock_gaia_ref(lambda request: self.identityMockCreateUpdateResponse("create"))
        impulses = CreateIdentityImpulse(str(uuid4()), str(""), {"de": "Deutsch"})
        result: CreatedIdentityImpulse = pipe(ops.first())(gaia_ref.preserve_create_identities([impulses])).run()
        self.assert_create_update_identity(result)

    def test_preserve_update_identity(self):
        gaia_ref = mock_gaia_ref(lambda request: self.identityMockCreateUpdateResponse("update"))
        impulses = UpdateIdentityImpulse(str(uuid4()), str(uuid4()), "qualifier", {"de": "Deutsch"})
        result: UpdatedIdentityImpulse = pipe(ops.first())(gaia_ref.preserve_update_identities([impulses])).run()
        self.assert_create_update_identity(result)

    def test_preserve_delete_identity(self):
        gaia_ref = mock_gaia_ref(lambda request: self.identityDeleteMockResponse())
        impulses = DeleteIdentityImpulse(str(uuid4()))
        result: DeletedIdentityImpulse = pipe(ops.first())(gaia_ref.preserve_delete_identities([impulses])).run()
        self.assert_delete_identity(result)

    def test_preserve_create_intent(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("create", "intents"))
        impulses = CreateIntentImpulse(str(uuid4()), "", "", dict(), list())
        result: CreatedIntentImpulse = pipe(ops.first())(gaia_ref.preserve_create_intents([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_update_intent(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("update", "intents"))
        impulses = UpdateIntentImpulse(str(uuid4()), str(uuid4()), "", "", dict(), list())
        result: UpdatedIntentImpulse = pipe(ops.first())(gaia_ref.preserve_update_intents([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_delete_intent(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityDeleteMockResponse("intents"))
        impulses = DeleteIntentImpulse(str(uuid4()), str(uuid4()))
        result: DeletedIntentImpulse = pipe(ops.first())(gaia_ref.preserve_delete_intents([impulses])).run()
        self.assert_delete_entity(result)

    def test_preserve_create_prompt(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("create", "prompts"))
        impulses = CreatePromptImpulse(str(uuid4()), "", "", dict(), list())
        result: CreatedPromptImpulse = pipe(ops.first())(gaia_ref.preserve_create_prompts([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_update_prompt(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("update", "prompts"))
        impulses = UpdatePromptImpulse(str(uuid4()), str(uuid4()), "", "", dict(), list())
        result: UpdatedPromptImpulse = pipe(ops.first())(gaia_ref.preserve_update_prompts([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_delete_prompt(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityDeleteMockResponse("prompts"))
        impulses = DeletePromptImpulse(str(uuid4()), str(uuid4()))
        result: DeletedPromptImpulse = pipe(ops.first())(gaia_ref.preserve_delete_prompts([impulses])).run()
        self.assert_delete_entity(result)

    def test_preserve_create_statement(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("create", "statements"))
        impulses = CreateStatementImpulse(str(uuid4()), "", "", dict(), list())
        result: CreatedStatementImpulse = pipe(ops.first())(gaia_ref.preserve_create_statements([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_update_statement(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("update", "statements"))
        impulses = UpdateStatementImpulse(str(uuid4()), str(uuid4()), "", "", dict(), list())
        result: UpdatedStatementImpulse = pipe(ops.first())(gaia_ref.preserve_update_statements([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_delete_statement(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityDeleteMockResponse("statements"))
        impulses = DeleteStatementImpulse(str(uuid4()), str(uuid4()))
        result: DeletedStatementImpulse = pipe(ops.first())(gaia_ref.preserve_delete_statements([impulses])).run()
        self.assert_delete_entity(result)

    def test_preserve_create_fulfilment(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("create", "fulfilments"))
        impulses = CreateFulfilmentImpulse(str(uuid4()), "", "", dict(), list())
        result: CreatedFulfilmentImpulse = pipe(ops.first())(gaia_ref.preserve_create_fulfilments([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_update_fulfilment(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityCreateUpdateMockResponse("update", "fulfilments"))
        impulses = UpdateFulfilmentImpulse(str(uuid4()), str(uuid4()), "", "", dict(), list())
        result: UpdatedFulfilmentImpulse = pipe(ops.first())(gaia_ref.preserve_update_fulfilments([impulses])).run()
        self.assert_create_update_entity(result)

    def test_preserve_delete_fulfilment(self):
        gaia_ref = mock_gaia_ref(lambda request: self.entityDeleteMockResponse("fulfilments"))
        impulses = DeleteFulfilmentImpulse(str(uuid4()), str(uuid4()))
        result: DeletedFulfilmentImpulse = pipe(ops.first())(gaia_ref.preserve_delete_fulfilments([impulses])).run()
        self.assert_delete_entity(result)

    def test_preserve_create_behaviour(self):
        gaia_ref = mock_gaia_ref(lambda request: self.behaviourCreateUpdateMockResponse("create"))
        impulses = CreateBehaviourImpulse(str(uuid4()), "", "", "", list())
        result: CreatedBehaviourImpulse = pipe(ops.first())(gaia_ref.preserve_create_behaviours([impulses])).run()
        self.assert_create_update_behaviour(result)

    def test_preserve_update_behaviour(self):
        gaia_ref = mock_gaia_ref(lambda request: self.behaviourCreateUpdateMockResponse("update"))
        impulses = UpdateBehaviourImpulse(str(uuid4()), str(uuid4()), "", "", "", list())
        result: UpdatedBehaviourImpulse = pipe(ops.first())(gaia_ref.preserve_update_behaviours([impulses])).run()
        self.assert_create_update_behaviour(result)

    def test_preserve_delete_behaviour(self):
        gaia_ref = mock_gaia_ref(lambda request: self.behaviourDeleteMockResponse())
        impulses = DeleteBehaviourImpulse(str(uuid4()), str(uuid4()))
        result: DeletedBehaviourImpulse = pipe(ops.first())(gaia_ref.preserve_delete_behaviours([impulses])).run()
        self.assert_delete_behaviour(result)

    def test_preserve_create_code(self):
        gaia_ref = mock_gaia_ref(lambda request: self.codeCreateUpdateMockResponse("create"))
        impulses = CreateCodeImpulse(str(uuid4()), "", "", dict(), "", list())
        result: CreatedCodeImpulse = pipe(ops.first())(gaia_ref.preserve_create_codes([impulses])).run()
        self.assert_create_update_code(result)

    def test_preserve_update_code(self):
        gaia_ref = mock_gaia_ref(lambda request: self.codeCreateUpdateMockResponse("update"))
        impulses = UpdateCodeImpulse(str(uuid4()), str(uuid4()), "", "", dict(), "", list())
        result: UpdatedCodeImpulse = pipe(ops.first())(gaia_ref.preserve_update_codes([impulses])).run()
        self.assert_create_update_code(result)

    def test_preserve_delete_code(self):
        gaia_ref = mock_gaia_ref(lambda request: self.codeDeleteMockResponse())
        impulses = DeleteCodeImpulse(str(uuid4()), str(uuid4()))
        result: DeletedCodeImpulse = pipe(ops.first())(gaia_ref.preserve_delete_codes([impulses])).run()
        self.assert_delete_code(result)

    def test_preserve_create_edge(self):
        gaia_ref = mock_gaia_ref(lambda request: self.edgeCreateMockResponse())
        impulses = CreateEdgeImpulse(str(uuid4()), str(uuid4()), "someType", 2.7, dict())
        result: CreatedEdgeImpulse = pipe(ops.first())(gaia_ref.preserve_create_edges([impulses])).run()
        self.assert_create_edge(result)

    def test_preserve_delete_edge(self):
        gaia_ref = mock_gaia_ref(lambda request: self.edgeDeleteMockResponse())
        impulses = DeleteEdgeImpulse(str(uuid4()), str(uuid4()))
        result: DeletedEdgeImpulse = pipe(ops.first())(gaia_ref.preserve_delete_edges([impulses])).run()
        self.assert_delete_edge(result)


if __name__ == '__main__':
    unittest.main()



from abc import abstractmethod, ABC

from gaia_sdk.api.SkillRef import SkillRef


class ISensorStream(ABC):
    @abstractmethod
    def data(self, uri: str):
        pass

    @abstractmethod
    def identity(self):
        pass

    @abstractmethod
    def skill(self, uri: str) -> SkillRef:
        pass





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class MetricsEntityCount(list):

    def intents(self):
        self.append(lambda x: "intents")

    def prompts(self):
        self.append(lambda x: "prompts")

    def statements(self):
        self.append(lambda x: "statements")

    def fulfilments(self):
        self.append(lambda x: "fulfilments")

    def behaviours(self):
        self.append(lambda x: "behaviours")

    def codes(self):
        self.append(lambda x: "codes")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



from typing import Callable, List

import requests
from rx.core.typing import Observable
from rx.scheduler import ThreadPoolScheduler

from gaia_sdk.api import ISensorStream, IdentityOp
from gaia_sdk.api.DataRef import DataRef
from gaia_sdk.api.GaiaCredentials import UsernamePasswordCredentials, GaiaCredentials, JWTCredentials
from gaia_sdk.api.ISensorFunction import ISensorFunction
from gaia_sdk.api.SkillRef import SkillRef
from gaia_sdk.graphql import RetrievalReq, ExperienceReq, KnowledgeReq, EdgeReq, \
    IntentReq, IdentityReq, PromptReq, StatementReq, FulfilmentReq, CodeReq, BehaviourReq, IntrospectionReq, \
    SkillIntrospectionReq, \
    PerceptionReq, PreservationReq, CreatedIdentityImpulse, UpdatedIdentityImpulse, DeletedIdentityImpulse, \
    CreatedIntentImpulse, UpdatedIntentImpulse, DeletedIntentImpulse, RetrievalRes, \
    ExperienceRes, EdgeRes, StatementRes, PromptRes, IntentRes, IdentityRes, KnowledgeRes, FulfilmentRes, CodeRes, \
    BehaviourRes, IntrospectionRes, SkillIntrospectionRes, PreservationRes, PerceptionRes, PerceivedImpulse, \
    PerceiveDataImpulse, PerceiveActionImpulse, DeleteIdentityImpulse, UpdateIdentityImpulse, CreateIdentityImpulse, \
    DeleteIntentImpulse, UpdateIntentImpulse, CreateIntentImpulse, \
    CreatePromptImpulse, UpdatePromptImpulse, DeletePromptImpulse, CreatedPromptImpulse, UpdatedPromptImpulse, \
    DeletedPromptImpulse, CreateStatementImpulse, UpdateStatementImpulse, DeleteStatementImpulse, \
    CreatedStatementImpulse, UpdatedStatementImpulse, DeletedStatementImpulse, CreateFulfilmentImpulse, \
    UpdateFulfilmentImpulse, DeleteFulfilmentImpulse, CreatedFulfilmentImpulse, UpdatedFulfilmentImpulse, \
    DeletedFulfilmentImpulse, CreateBehaviourImpulse, UpdateBehaviourImpulse, DeleteBehaviourImpulse, \
    CreatedBehaviourImpulse, UpdatedBehaviourImpulse, DeletedBehaviourImpulse, CreateCodeImpulse, UpdateCodeImpulse, \
    DeleteCodeImpulse, CreatedCodeImpulse, UpdatedCodeImpulse, DeletedCodeImpulse, CreateEdgeImpulse, \
    DeleteEdgeImpulse, CreatedEdgeImpulse, DeletedEdgeImpulse, BehaviourExecutionRes, BehaviourExecutionReq, \
    BehaviourExecutionDetailReq, BehaviourExecutionDetailRes, \
    SkillProvisionBuildJobReq, SkillProvisionBuildJobRes

from gaia_sdk.http.HttpSensorFunction import HttpSensorFunction
from gaia_sdk.http.HttpSensorStream import HttpSensorStream
from gaia_sdk.http.response.LoggedIn import LoggedIn
from gaia_sdk.graphql.GaiaClientFactory import GaiaClientFactory
from gaia_sdk.http.GaiaStreamClientBuilder import GaiaStreamClientFactory

Uuid = str


class Gaia:
    _pool = ThreadPoolScheduler(5)
    _client_factory = GaiaClientFactory()
    _stream_client_factory = GaiaStreamClientFactory()

    @classmethod
    def connect(cls, url: str, credentials: GaiaCredentials) -> 'GaiaRef':
        config = GaiaConfig(url, HttpSensorFunction(url, credentials, cls._pool, cls._client_factory),
                            HttpSensorStream(url, credentials, cls._pool, cls._stream_client_factory))
        return GaiaRef(config, config.functionProcessor, config.streamProcessor)

    @classmethod
    def login(cls, url: str, credentials: UsernamePasswordCredentials) -> 'GaiaRef':
        headers = {'Content-Type': 'application/json'}
        response = requests.post(f"{url}/api/auth/access", json=credentials.__repr__(), headers=headers)
        response.raise_for_status()
        return Gaia.connect(url, JWTCredentials(LoggedIn(response.json()).access_token))

class GaiaConfig:
    url: str
    functionProcessor: ISensorFunction
    streamProcessor: ISensorStream

    def __init__(self, url: str, functionProcessor: ISensorFunction, streamProcessor: ISensorStream):
        self.url = url
        self.functionProcessor = functionProcessor
        self.streamProcessor = streamProcessor

    def __eq__(self, other):
        return self.url == other.url

    def __repr__(self):
        return {'url': self.url}


class GaiaRef(ISensorFunction):  # TODO: implement ISensorStream
    config: GaiaConfig
    f_proc: ISensorFunction
    s_proc: ISensorStream

    def __init__(self, config: GaiaConfig, f_proc: ISensorFunction, s_proc: ISensorStream):
        self.config = config
        self.f_proc = f_proc
        self.s_proc = s_proc

    def __eq__(self, other):
        return self.config == other.config

    def __repr__(self):
        return {'config': self.config}

    def data(self, uri: str) -> DataRef:
        return self.s_proc.data(uri)

    def identity(self) -> IdentityOp:
        return self.s_proc.identity()

    def skill(self, url: str) -> SkillRef:
        return self.s_proc.skill(url)

    def retrieve(self, config: Callable[[RetrievalReq], None]) -> Observable[RetrievalRes]:
        return self.f_proc.retrieve(config)

    def retrieve_experience(self, config: Callable[[ExperienceReq], None]) -> Observable[ExperienceRes]:
        return self.f_proc.retrieve_experience(config)

    def retrieve_knowledge(self, config: Callable[[KnowledgeReq], None]) -> Observable[KnowledgeRes]:
        return self.f_proc.retrieve_knowledge(config)

    def retrieve_edges(self, source: Uuid, config: Callable[[EdgeReq], None], limit: int = None, offset: int = None) -> \
            Observable[EdgeRes]:
        return self.f_proc.retrieve_edges(source, config, limit, offset)

    def retrieve_edge(self, source: Uuid, edgeId: Uuid, config: Callable[[EdgeReq], None]) -> Observable[EdgeRes]:
        return self.f_proc.retrieve_edge(source, edgeId, config)

    def retrieve_identities(self, config: Callable[[IdentityReq], None], limit: int = None, offset: int = None) -> \
            Observable[IdentityRes]:
        return self.f_proc.retrieve_identities(config, limit, offset)

    def retrieve_identity(self, identityId: Uuid, config: Callable[[IdentityReq], None]) -> Observable[IdentityRes]:
        return self.f_proc.retrieve_identity(identityId, config)

    def retrieve_intents(self, identityId: Uuid, config: Callable[[IntentReq], None], limit: int = None,
                         offset: int = None) -> Observable[IntentRes]:
        return self.f_proc.retrieve_intents(identityId, config, limit, offset)

    def retrieve_intent(self, identityId: Uuid, reference: Uuid, config: Callable[[IntentReq], None]) -> Observable[
        IntentRes]:
        return self.f_proc.retrieve_intent(identityId, reference, config)

    def retrieve_prompts(self, identityId: Uuid, config: Callable[[PromptReq], None], limit: int = None,
                         offset: int = None) -> Observable[PromptRes]:
        return self.f_proc.retrieve_prompts(identityId, config, limit, offset)

    def retrieve_prompt(self, identityId: Uuid, reference: Uuid, config: Callable[[PromptReq], None]) -> Observable[
        PromptRes]:
        return self.f_proc.retrieve_prompt(identityId, reference, config)

    def retrieve_statements(self, identityId: Uuid, config: Callable[[StatementReq], None], limit: int = None,
                            offset: int = None) -> Observable[StatementRes]:
        return self.f_proc.retrieve_statements(identityId, config, limit, offset)

    def retrieve_statement(self, identityId: Uuid, reference: Uuid, config: Callable[[StatementReq], None]) -> \
            Observable[StatementRes]:
        return self.f_proc.retrieve_statement(identityId, reference, config)

    def retrieve_fulfilments(self, identityId: Uuid, config: Callable[[FulfilmentReq], None], limit: int = None,
                             offset: int = None) -> Observable[FulfilmentRes]:
        return self.f_proc.retrieve_fulfilments(identityId, config, limit, offset)

    def retrieve_fulfilment(self, identityId: Uuid, reference: Uuid, config: Callable[[FulfilmentReq], None]) -> \
            Observable[FulfilmentRes]:
        return self.f_proc.retrieve_fulfilment(identityId, reference, config)

    def retrieve_codes(self, identityId: Uuid, config: Callable[[CodeReq], None], limit: int = None,
                       offset: int = None) -> Observable[CodeRes]:
        return self.f_proc.retrieve_codes(identityId, config, limit, offset)

    def retrieve_code(self, identityId: Uuid, reference: Uuid, config: Callable[[CodeReq], None]) -> Observable[
        CodeRes]:
        return self.f_proc.retrieve_code(identityId, reference, config)

    def retrieve_behaviours(self, identityId: Uuid, config: Callable[[BehaviourReq], None], limit: int = None,
                            offset: int = None) -> Observable[BehaviourRes]:
        return self.f_proc.retrieve_behaviours(identityId, config, limit, offset)

    def retrieve_behaviour(self, identityId: Uuid, reference: Uuid, config: Callable[[BehaviourReq], None]) -> \
            Observable[BehaviourRes]:
        return self.f_proc.retrieve_behaviour(identityId, reference, config)

    def retrieve_behaviour_executions(self, identity_id: Uuid, config: Callable[[BehaviourExecutionReq], None], limit: int = None, offset: int = None, startDate: str = None, endDate: str = None) \
            -> Observable[BehaviourExecutionRes]:
        return self.f_proc.retrieve_behaviour_executions(identity_id, config, limit, offset, startDate, endDate)

    def retrieve_behaviour_execution(self, identity_id: Uuid, process_instance_id: Uuid, config: Callable[[BehaviourExecutionDetailReq], None]) -> Observable[BehaviourExecutionDetailRes]:
        return self.f_proc.retrieve_behaviour_execution(identity_id, process_instance_id, config)

    def retrieve_skill_provision_build_jobs(self, tenant_id: Uuid, config: Callable[[SkillProvisionBuildJobReq], None]) -> Observable[SkillProvisionBuildJobRes]:
        return self.f_proc.retrieve_skill_provision_build_jobs(tenant_id, config)

    def introspect(self, config: Callable[[IntrospectionReq], None]) -> Observable[IntrospectionRes]:
        return self.f_proc.introspect(config)

    def introspect_skills(self, config: Callable[[SkillIntrospectionReq], None]) -> Observable[SkillIntrospectionRes]:
        return self.f_proc.introspect_skills(config)

    def preserve(self, config: Callable[[PreservationReq], None]) -> Observable[PreservationRes]:
        return self.f_proc.preserve(config)

    def preserve_create_identities(self, impulses: List[CreateIdentityImpulse]) -> Observable[CreatedIdentityImpulse]:
        return self.f_proc.preserve_create_identities(impulses)

    def preserve_update_identities(self, impulses: List[UpdateIdentityImpulse]) -> Observable[UpdatedIdentityImpulse]:
        return self.f_proc.preserve_update_identities(impulses)

    def preserve_delete_identities(self, impulses: List[DeleteIdentityImpulse]) -> Observable[DeletedIdentityImpulse]:
        return self.f_proc.preserve_delete_identities(impulses)

    def preserve_create_intents(self, impulses: List[CreateIntentImpulse]) -> Observable[CreatedIntentImpulse]:
        return self.f_proc.preserve_create_intents(impulses)

    def preserve_update_intents(self, impulses: List[UpdateIntentImpulse]) -> Observable[UpdatedIntentImpulse]:
        return self.f_proc.preserve_update_intents(impulses)

    def preserve_delete_intents(self, impulses: List[DeleteIntentImpulse]) -> Observable[DeletedIntentImpulse]:
        return self.f_proc.preserve_delete_intents(impulses)

    def preserve_create_prompts(self, impulses: List[CreatePromptImpulse]) -> Observable[CreatedPromptImpulse]:
        return self.f_proc.preserve_create_prompts(impulses)

    def preserve_update_prompts(self, impulses: List[UpdatePromptImpulse]) -> Observable[UpdatedPromptImpulse]:
        return self.f_proc.preserve_update_prompts(impulses)

    def preserve_delete_prompts(self, impulses: List[DeletePromptImpulse]) -> Observable[DeletedPromptImpulse]:
        return self.f_proc.preserve_delete_prompts(impulses)

    def preserve_create_statements(self, impulses: List[CreateStatementImpulse]) -> Observable[CreatedStatementImpulse]:
        return self.f_proc.preserve_create_statements(impulses)

    def preserve_update_statements(self, impulses: List[UpdateStatementImpulse]) -> Observable[UpdatedStatementImpulse]:
        return self.f_proc.preserve_update_statements(impulses)

    def preserve_delete_statements(self, impulses: List[DeleteStatementImpulse]) -> Observable[DeletedStatementImpulse]:
        return self.f_proc.preserve_delete_statements(impulses)

    def preserve_create_fulfilments(self, impulses: List[CreateFulfilmentImpulse]) -> Observable[
        CreatedFulfilmentImpulse]:
        return self.f_proc.preserve_create_fulfilments(impulses)

    def preserve_update_fulfilments(self, impulses: List[UpdateFulfilmentImpulse]) -> Observable[
        UpdatedFulfilmentImpulse]:
        return self.f_proc.preserve_update_fulfilments(impulses)

    def preserve_delete_fulfilments(self, impulses: List[DeleteFulfilmentImpulse]) -> Observable[
        DeletedFulfilmentImpulse]:
        return self.f_proc.preserve_delete_fulfilments(impulses)

    def preserve_create_behaviours(self, impulses: List[CreateBehaviourImpulse]) -> Observable[CreatedBehaviourImpulse]:
        return self.f_proc.preserve_create_behaviours(impulses)

    def preserve_update_behaviours(self, impulses: List[UpdateBehaviourImpulse]) -> Observable[UpdatedBehaviourImpulse]:
        return self.f_proc.preserve_update_behaviours(impulses)

    def preserve_delete_behaviours(self, impulses: List[DeleteBehaviourImpulse]) -> Observable[DeletedBehaviourImpulse]:
        return self.f_proc.preserve_delete_behaviours(impulses)

    def preserve_create_codes(self, impulses: List[CreateCodeImpulse]) -> Observable[CreatedCodeImpulse]:
        return self.f_proc.preserve_create_codes(impulses)

    def preserve_update_codes(self, impulses: List[UpdateCodeImpulse]) -> Observable[UpdatedCodeImpulse]:
        return self.f_proc.preserve_update_codes(impulses)

    def preserve_delete_codes(self, impulses: List[DeleteCodeImpulse]) -> Observable[DeletedCodeImpulse]:
        return self.f_proc.preserve_delete_codes(impulses)

    def preserve_create_edges(self, impulses: List[CreateEdgeImpulse]) -> Observable[CreatedEdgeImpulse]:
        return self.f_proc.preserve_create_edges(impulses)

    def preserve_delete_edges(self, impulses: List[DeleteEdgeImpulse]) -> Observable[DeletedEdgeImpulse]:
        return self.f_proc.preserve_delete_edges(impulses)

    def perceive(self, config: Callable[[PerceptionReq], None]) -> Observable[PerceptionRes]:
        return self.f_proc.perceive(config)

    def perceive_action(self, impulse: PerceiveActionImpulse) -> Observable[PerceivedImpulse]:
        return self.f_proc.perceive_action(impulse)

    def perceive_data(self, impulse: PerceiveDataImpulse) -> Observable[PerceivedImpulse]:
        return self.f_proc.perceive_data(impulse)





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class PerceiveActionImpulse():
    """
    Input for action perception impulse
    """
    broadcast: bool
    identityId: str
    type: str
    data: dict

    def __init__(self, broadcast: bool, identityId: str, type: str, data: dict):
        self.broadcast = broadcast
        self.identityId = identityId
        self.type = type
        self.data = data

    def __eq__(self, other):
        if type(other) is type(self):
            return self.broadcast == other.broadcast and self.identityId == other.identityId and self.type == other.type and self.data == other.data
        return False

    def __repr__(self):
        return {'broadcast': self.broadcast, 'identityId': self.identityId, 'type': self.type, 'data': self.data}





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class BuildInEvaluation(list):

    def behaviour(self):
        self.append(lambda x: "behaviour")

    def gaiaquery(self):
        self.append(lambda x: "gaiaquery")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.request.type.TopExecutedBehaviour import TopExecutedBehaviour
from gaia_sdk.graphql.request.type.BehaviourState import BehaviourState
from gaia_sdk.graphql.request.type.MetricsEntityCount import MetricsEntityCount

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class IdentityMetrics(list):
    """
    Represents identity metrics information
    """

    def identity_id(self):
        self.append(lambda x: "identityId")

    def entity_count(self, config: Callable[['MetricsEntityCount'], None]):
        def callback(registry: VariableRegistry):
            entity = MetricsEntityCount()
            config(entity)
            return "entity_count {" + entity.render(registry) + "}"
        self.append(callback)

    def top_executed_behaviours(self, config: Callable[['TopExecutedBehaviour'], None]):
        def callback(registry: VariableRegistry):
            entity = TopExecutedBehaviour()
            config(entity)
            return "top_executed_behaviours {" + entity.render(registry) + "}"
        self.append(callback)

    def behaviour_states(self, config: Callable[['BehaviourState'], None]):
        def callback(registry: VariableRegistry):
            entity = BehaviourState()
            config(entity)
            return "behaviour_states {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



from typing import Callable, List

import rx
from rx.core.typing import Observable, Scheduler

from gaia_sdk.api.ISensorFunction import ISensorFunction
from gaia_sdk.api.rx.rx import mapQ, mapM, flat_mapM, flat_mapQ
from gaia_sdk.graphql import QueryReq, QueryRes, RetrievalReq, ExperienceReq, KnowledgeReq, EdgeReq, \
    IntentReq, IdentityReq, PromptReq, StatementReq, FulfilmentReq, CodeReq, BehaviourReq, IntrospectionReq, \
    SkillIntrospectionReq, \
    PerceptionReq, PreservationReq, CreatedIdentityImpulse, UpdatedIdentityImpulse, DeletedIdentityImpulse, \
    CreatedIntentImpulse, UpdatedIntentImpulse, DeletedIntentImpulse, RetrievalRes, \
    ExperienceRes, EdgeRes, StatementRes, PromptRes, IntentRes, IdentityRes, KnowledgeRes, FulfilmentRes, CodeRes, \
    BehaviourRes, IntrospectionRes, SkillIntrospectionRes, PreservationRes, PerceptionRes, PerceivedImpulse, \
    MutationReq, MutationRes, PerceiveDataImpulse, PerceiveActionImpulse, DeleteIdentityImpulse, \
    UpdateIdentityImpulse, CreateIdentityImpulse, DeleteIntentImpulse, \
    UpdateIntentImpulse, CreateIntentImpulse, CreatePromptImpulse, UpdatePromptImpulse, DeletePromptImpulse, \
    CreatedPromptImpulse, UpdatedPromptImpulse, \
    DeletedPromptImpulse, CreateStatementImpulse, UpdateStatementImpulse, DeleteStatementImpulse, \
    CreatedStatementImpulse, UpdatedStatementImpulse, DeletedStatementImpulse, CreateFulfilmentImpulse, \
    UpdateFulfilmentImpulse, DeleteFulfilmentImpulse, CreatedFulfilmentImpulse, UpdatedFulfilmentImpulse, \
    DeletedFulfilmentImpulse, CreateBehaviourImpulse, UpdateBehaviourImpulse, DeleteBehaviourImpulse, \
    CreatedBehaviourImpulse, UpdatedBehaviourImpulse, DeletedBehaviourImpulse, CreateCodeImpulse, UpdateCodeImpulse, \
    DeleteCodeImpulse, CreatedCodeImpulse, UpdatedCodeImpulse, DeletedCodeImpulse, CreateEdgeImpulse, \
    DeleteEdgeImpulse, CreatedEdgeImpulse, DeletedEdgeImpulse, BehaviourExecutionRes, BehaviourExecutionReq, \
    BehaviourExecutionDetailReq, BehaviourExecutionDetailRes, \
    SkillProvisionBuildJobReq, SkillProvisionBuildJobRes

from gaia_sdk.graphql.GaiaClientFactory import GaiaClientFactory
from gaia_sdk.graphql.GaiaRequest import GaiaRequest

Uuid = str


class HttpSensorFunction(ISensorFunction):

    def __init__(self, url: str, credentials, scheduler: Scheduler, client_factory: GaiaClientFactory):
        self.client = client_factory.http(url + "/api/entity").with_credentials(credentials).build()
        self._scheduler = scheduler

    def retrieve(self, config: Callable[[RetrievalReq], None]) -> Observable[RetrievalRes]:
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(config)
        query_res: Callable[[QueryRes], RetrievalRes] = lambda x: x.retrieve

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_experience(self, config: Callable[[ExperienceReq], None]) -> Observable[ExperienceRes]:
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.experience(config)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], ExperienceRes] = lambda x: x.retrieve.experience

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_knowledge(self, config: Callable[[KnowledgeReq], None]) -> Observable[KnowledgeRes]:
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(config)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], KnowledgeRes] = lambda x: x.retrieve.knowledge

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_edges(self, source: Uuid, config: Callable[[EdgeReq], None], limit: int, offset: int) -> Observable[
        EdgeRes]:
        knowledge_req: Callable[[KnowledgeReq], None] = lambda x: x.edges(source, limit, offset, None, None, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(knowledge_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], EdgeRes] = lambda x: x.retrieve.knowledge.edges

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_edge(self, source: Uuid, edgeId: Uuid, config: Callable[[EdgeReq], None]) -> Observable[EdgeRes]:
        knowledge_req: Callable[[KnowledgeReq], None] = lambda x: x.edge(source, edgeId, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(knowledge_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], EdgeRes] = lambda x: x.retrieve.knowledge.edge

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_identities(self, config: Callable[[IdentityReq], None], limit: int, offset: int) -> Observable[
        IdentityRes]:
        identities_req: Callable[[IdentityReq], None] = lambda x: x.identities(limit, offset, None, None, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(identities_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], IdentityRes] = lambda x: x.retrieve.knowledge.identities

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_identity(self, identityId: Uuid, config: Callable[[IdentityReq], None]) -> Observable[IdentityRes]:
        identities_req: Callable[[IdentityReq], None] = lambda x: x.identity(identityId, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(identities_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], IdentityRes] = lambda x: x.retrieve.knowledge.identity

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_intents(self, identityId: Uuid, config: Callable[[IntentReq], None], limit: int, offset: int) -> \
            Observable[IntentRes]:
        intents_req: Callable[[IntentReq], None] = lambda x: x.intents(identityId, limit, offset, None, None, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(intents_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], IntentRes] = lambda x: x.retrieve.knowledge.intents

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_intent(self, identityId: Uuid, reference: Uuid, config: Callable[[IntentReq], None]) -> Observable[
        IntentRes]:
        intents_req: Callable[[IntentReq], None] = lambda x: x.intent(identityId, reference, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(intents_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], IntentRes] = lambda x: x.retrieve.knowledge.intent

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_prompts(self, identityId: Uuid, config: Callable[[PromptReq], None], limit: int, offset: int) -> \
            Observable[PromptRes]:
        prompts_req: Callable[[PromptReq], None] = lambda x: x.prompts(identityId, limit, offset, None, None, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(prompts_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], PromptRes] = lambda x: x.retrieve.knowledge.prompts

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_prompt(self, identityId: Uuid, reference: Uuid, config: Callable[[PromptReq], None]) -> Observable[
        PromptRes]:
        prompts_req: Callable[[PromptReq], None] = lambda x: x.prompt(identityId, reference, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(prompts_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], PromptRes] = lambda x: x.retrieve.knowledge.prompt

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_statements(self, identityId: Uuid, config: Callable[[StatementReq], None], limit: int, offset: int) -> \
            Observable[StatementRes]:
        statement_req: Callable[[StatementReq], None] = lambda x: x.statements(identityId, limit, offset, None, None,
                                                                               config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(statement_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], StatementRes] = lambda x: x.retrieve.knowledge.statements

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_statement(self, identityId: Uuid, reference: Uuid, config: Callable[[StatementReq], None]) -> \
            Observable[StatementRes]:
        statement_req: Callable[[StatementReq], None] = lambda x: x.statement(identityId, reference, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(statement_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], StatementRes] = lambda x: x.retrieve.knowledge.statement

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_fulfilments(self, identityId: Uuid, config: Callable[[FulfilmentReq], None], limit: int,
                             offset: int) -> Observable[FulfilmentRes]:
        fulfilment_req: Callable[[FulfilmentReq], None] = lambda x: x.fulfilments(identityId, limit, offset, None, None,
                                                                                  config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(fulfilment_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], FulfilmentRes] = lambda x: x.retrieve.knowledge.fulfilments

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_fulfilment(self, identityId: Uuid, reference: Uuid, config: Callable[[FulfilmentReq], None]) -> \
            Observable[FulfilmentRes]:
        fulfilment_req: Callable[[FulfilmentReq], None] = lambda x: x.fulfilment(identityId, reference, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(fulfilment_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], FulfilmentRes] = lambda x: x.retrieve.knowledge.fulfilment

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_codes(self, identityId: Uuid, config: Callable[[CodeReq], None], limit: int, offset: int) -> \
            Observable[CodeRes]:
        code_req: Callable[[CodeReq], None] = lambda x: x.codes(identityId, limit, offset, None, None, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(code_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], CodeRes] = lambda x: x.retrieve.knowledge.codes

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_code(self, identityId: Uuid, reference: Uuid, config: Callable[[CodeReq], None]) -> Observable[
        CodeRes]:
        code_req: Callable[[CodeReq], None] = lambda x: x.code(identityId, reference, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(code_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], CodeRes] = lambda x: x.retrieve.knowledge.code

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_behaviours(self, identityId: Uuid, config: Callable[[BehaviourReq], None], limit: int, offset: int) -> \
            Observable[BehaviourRes]:
        behaviour_req: Callable[[BehaviourReq], None] = lambda x: x.behaviours(identityId, limit, offset, None, None,
                                                                               config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(behaviour_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], BehaviourRes] = lambda x: x.retrieve.knowledge.behaviours

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_behaviour(self, identityId: Uuid, reference: Uuid, config: Callable[[BehaviourReq], None]) -> \
            Observable[BehaviourRes]:
        behaviour_req: Callable[[BehaviourReq], None] = lambda x: x.behaviour(identityId, reference, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(behaviour_req)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], BehaviourRes] = lambda x: x.retrieve.knowledge.behaviour

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def retrieve_behaviour_executions(self, identity_id: Uuid, config: Callable[[BehaviourExecutionReq], None], limit: int = None, offset: int = None, startDate: str = None, endDate: str = None) \
        -> Observable[BehaviourExecutionRes]:
        executions_req: Callable[[BehaviourExecutionReq], None] = lambda x: x.behaviour_executions(identity_id, limit, offset, startDate, endDate, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.experience(executions_req)

        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], BehaviourExecutionRes] = lambda x: x.retrieve.experience.behaviour_executions

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def retrieve_behaviour_execution(self, identity_id: Uuid, process_instance_id: Uuid, config: Callable[[BehaviourExecutionDetailReq], None]) \
            -> Observable[BehaviourExecutionDetailRes]:

        behaviour_exec_query: Callable[[BehaviourExecutionDetailRes], None] = lambda x: x.behaviour_execution(identity_id, process_instance_id, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.experience(behaviour_exec_query)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res: Callable[[QueryRes], BehaviourExecutionDetailRes] = lambda x: x.retrieve.experience.behaviour_execution

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)


    def retrieve_skill_provision_build_jobs(self, tenant_id: Uuid, config: Callable[[SkillProvisionBuildJobReq], None]) -> Observable[SkillProvisionBuildJobRes]:
        skill_build_jobs_query: Callable[[ExperienceReq], None] = lambda x: x.skill_provision_build_jobs(tenant_id, config)
        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.experience(skill_build_jobs_query)
        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
        query_res = lambda x: x.retrieve.experience.skill_provision_build_jobs
        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return flat_mapQ(observable, query_res)

    def introspect(self, config: Callable[[IntrospectionReq], None]) -> Observable[IntrospectionRes]:
        query_req: Callable[[QueryReq], None] = lambda x: x.introspect(config)
        query_res: Callable[[QueryRes], IntrospectionRes] = lambda x: x.introspect

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def introspect_skills(self, config: Callable[[SkillIntrospectionReq], None]) -> Observable[SkillIntrospectionRes]:
        introspect_skills: Callable[[IntrospectionReq], None] = lambda x: x.skills(config)
        query_req: Callable[[QueryReq], None] = lambda x: x.introspect(introspect_skills)
        query_res: Callable[[QueryRes], SkillIntrospectionRes] = lambda x: x.introspect.skills

        observable = rx.from_callable(lambda: self.client.query(GaiaRequest.query(query_req)), self._scheduler)
        return mapQ(observable, query_res)

    def preserve(self, config: Callable[[PreservationReq], None]) -> Observable[PreservationRes]:
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(config)
        mutation_res: Callable[[MutationRes], PreservationRes] = lambda x: x.preserve

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return mapM(observable, mutation_res)

    def preserve_create_identities(self, impulses: List[CreateIdentityImpulse]) -> Observable[CreatedIdentityImpulse]:
        def data_req(x):
            x.identity_id()
            x.tenant_id()
            x.qualifier()
            x.available_languages()

        def identities_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(
            lambda e: e.identities(impulses, identities_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedIdentityImpulse] = lambda x: x.preserve.create.identities

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_update_identities(self, impulses: List[UpdateIdentityImpulse]) -> Observable[UpdatedIdentityImpulse]:
        def data_req(x):
            x.identity_id()
            x.tenant_id()
            x.qualifier()
            x.available_languages()

        def identities_req(x):
            x.id()
            x.data(data_req)

        update_identities: Callable[[PreservationReq], None] = lambda x: x.update(
            lambda e: e.identities(impulses, identities_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(update_identities)
        mutation_res: Callable[[MutationRes], UpdatedIdentityImpulse] = lambda x: x.preserve.update.identities

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_identities(self, impulses: List[DeleteIdentityImpulse]) -> Observable[DeletedIdentityImpulse]:
        def data_req(x):
            x.identity_id()

        def identities_req(x):
            x.id()
            x.data(data_req)

        delete_identities: Callable[[PreservationReq], None] = lambda x: x.delete(
            lambda e: e.identities(impulses, identities_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_identities)
        mutation_res: Callable[[MutationRes], DeletedIdentityImpulse] = lambda x: x.preserve.delete.identities

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_create_intents(self, impulses: List[CreateIntentImpulse]) -> Observable[CreatedIntentImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def intent_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(lambda e: e.intents(impulses, intent_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedIntentImpulse] = lambda x: x.preserve.create.intents

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_update_intents(self, impulses: List[UpdateIntentImpulse]) -> Observable[UpdatedIntentImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def intent_req(x):
            x.id()
            x.data(data_req)

        update_intents: Callable[[PreservationReq], None] = lambda x: x.update(
            lambda e: e.intents(impulses, intent_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(update_intents)
        mutation_res: Callable[[MutationRes], UpdatedIntentImpulse] = lambda x: x.preserve.update.intents

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_intents(self, impulses: List[DeleteIntentImpulse]) -> Observable[DeletedIntentImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()

        def intent_req(x):
            x.id()
            x.data(data_req)
        delete_intents: Callable[[PreservationReq], None] = lambda x: x.delete(
            lambda e: e.intents(impulses, intent_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_intents)
        mutation_res: Callable[[MutationRes], DeletedIntentImpulse] = lambda x: x.preserve.delete.intents

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_create_prompts(self, impulses: List[CreatePromptImpulse]) -> Observable[CreatedPromptImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def prompt_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(lambda e: e.prompts(impulses, prompt_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedPromptImpulse] = lambda x: x.preserve.create.prompts

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_update_prompts(self, impulses: List[UpdatePromptImpulse]) -> Observable[UpdatedPromptImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def prompt_req(x):
            x.id()
            x.data(data_req)

        update_prompts: Callable[[PreservationReq], None] = lambda x: x.update(
            lambda e: e.prompts(impulses, prompt_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(update_prompts)
        mutation_res: Callable[[MutationRes], UpdatedPromptImpulse] = lambda x: x.preserve.update.prompts

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_prompts(self, impulses: List[DeletePromptImpulse]) -> Observable[DeletedPromptImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()

        def prompt_req(x):
            x.id()
            x.data(data_req)

        delete_prompts: Callable[[PreservationReq], None] = lambda x: x.delete(
            lambda e: e.prompts(impulses, prompt_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_prompts)
        mutation_res: Callable[[MutationRes], DeletedPromptImpulse] = lambda x: x.preserve.delete.prompts

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_create_statements(self, impulses: List[CreateStatementImpulse]) -> Observable[CreatedStatementImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def statement_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(
            lambda e: e.statements(impulses, statement_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedStatementImpulse] = lambda x: x.preserve.create.statements

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_update_statements(self, impulses: List[UpdateStatementImpulse]) -> Observable[UpdatedStatementImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def statement_req(x):
            x.id()
            x.data(data_req)

        update_statements: Callable[[PreservationReq], None] = lambda x: x.update(
            lambda e: e.statements(impulses, statement_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(update_statements)
        mutation_res: Callable[[MutationRes], UpdatedStatementImpulse] = lambda x: x.preserve.update.statements

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_statements(self, impulses: List[DeleteStatementImpulse]) -> Observable[DeletedStatementImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()

        def statement_req(x):
            x.id()
            x.data(data_req)

        delete_statements: Callable[[PreservationReq], None] = lambda x: x.delete(
            lambda e: e.statements(impulses, statement_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_statements)
        mutation_res: Callable[[MutationRes], DeletedStatementImpulse] = lambda x: x.preserve.delete.statements

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_create_fulfilments(self, impulses: List[CreateFulfilmentImpulse]) -> Observable[CreatedFulfilmentImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def fulfilment_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(
            lambda e: e.fulfilments(impulses, fulfilment_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedFulfilmentImpulse] = lambda x: x.preserve.create.fulfilments

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_update_fulfilments(self, impulses: List[UpdateFulfilmentImpulse]) -> Observable[UpdatedFulfilmentImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.utterance()
            x.label_list()
            x.version()

        def fulfilment_req(x):
            x.id()
            x.data(data_req)

        update_fulfilments: Callable[[PreservationReq], None] = lambda x: x.update(
            lambda e: e.fulfilments(impulses, fulfilment_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(update_fulfilments)
        mutation_res: Callable[[MutationRes], UpdatedFulfilmentImpulse] = lambda x: x.preserve.update.fulfilments

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_fulfilments(self, impulses: List[DeleteFulfilmentImpulse]) -> Observable[DeletedFulfilmentImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()

        def fulfilment_req(x):
            x.id()
            x.data(data_req)

        delete_fulfilments: Callable[[PreservationReq], None] = lambda x: x.delete(
            lambda e: e.fulfilments(impulses, fulfilment_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_fulfilments)
        mutation_res: Callable[[MutationRes], DeletedFulfilmentImpulse] = lambda x: x.preserve.delete.fulfilments

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_create_behaviours(self, impulses: List[CreateBehaviourImpulse]) -> Observable[CreatedBehaviourImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.label_list()

        def behaviour_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(
            lambda e: e.behaviours(impulses, behaviour_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedBehaviourImpulse] = lambda x: x.preserve.create.behaviours

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_update_behaviours(self, impulses: List[UpdateBehaviourImpulse]) -> Observable[UpdatedBehaviourImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.label_list()

        def behaviour_req(x):
            x.id()
            x.data(data_req)

        update_behaviours: Callable[[PreservationReq], None] = lambda x: x.update(
            lambda e: e.behaviours(impulses, behaviour_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(update_behaviours)
        mutation_res: Callable[[MutationRes], UpdatedBehaviourImpulse] = lambda x: x.preserve.update.behaviours

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_behaviours(self, impulses: List[DeleteBehaviourImpulse]) -> Observable[DeletedBehaviourImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()

        def behaviour_req(x):
            x.id()
            x.data(data_req)

        delete_behaviours: Callable[[PreservationReq], None] = lambda x: x.delete(
            lambda e: e.behaviours(impulses, behaviour_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_behaviours)
        mutation_res: Callable[[MutationRes], DeletedBehaviourImpulse] = lambda x: x.preserve.delete.behaviours

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_create_codes(self, impulses: List[CreateCodeImpulse]) -> Observable[CreatedCodeImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.code()
            x.type()
            x.label_list()

        def code_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(lambda e: e.codes(impulses, code_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedCodeImpulse] = lambda x: x.preserve.create.codes

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_update_codes(self, impulses: List[UpdateCodeImpulse]) -> Observable[UpdatedCodeImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()
            x.qualifier()
            x.appendent()
            x.code()
            x.type()
            x.label_list()

        def code_req(x):
            x.id()
            x.data(data_req)

        update_codes: Callable[[PreservationReq], None] = lambda x: x.update(lambda e: e.codes(impulses, code_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(update_codes)
        mutation_res: Callable[[MutationRes], UpdatedCodeImpulse] = lambda x: x.preserve.update.codes

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_codes(self, impulses: List[DeleteCodeImpulse]) -> Observable[DeletedCodeImpulse]:
        def data_req(x):
            x.identity_id()
            x.reference()

        def code_req(x):
            x.id()
            x.data(data_req)

        delete_codes: Callable[[PreservationReq], None] = lambda x: x.delete(lambda e: e.codes(impulses, code_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_codes)
        mutation_res: Callable[[MutationRes], DeletedCodeImpulse] = lambda x: x.preserve.delete.codes

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_create_edges(self, impulses: List[CreateEdgeImpulse]) -> Observable[CreatedEdgeImpulse]:
        def data_req(x):
            x.source()
            x.target()
            x.type()
            x.weight()
            x.edge_id()
            x.properties()

        def edge_req(x):
            x.id()
            x.data(data_req)

        create_req: Callable[[PreservationReq], None] = lambda x: x.create(lambda e: e.edges(impulses, edge_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
        mutation_res: Callable[[MutationRes], CreatedEdgeImpulse] = lambda x: x.preserve.create.edges

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def preserve_delete_edges(self, impulses: List[DeleteEdgeImpulse]) -> Observable[DeletedEdgeImpulse]:
        def data_req(x):
            x.source()
            x.edge_id()

        def edge_req(x):
            x.id()
            x.data(data_req)

        delete_edges: Callable[[PreservationReq], None] = lambda x: x.delete(lambda e: e.edges(impulses, edge_req))
        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_edges)
        mutation_res: Callable[[MutationRes], DeletedEdgeImpulse] = lambda x: x.preserve.delete.edges

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return flat_mapM(observable, mutation_res)

    def perceive(self, config: Callable[[PerceptionReq], None]) -> Observable[PerceptionRes]:
        mutation_req: Callable[[MutationReq], None] = lambda x: x.perceive(config)
        mutation_res: Callable[[MutationRes], PerceptionRes] = lambda x: x.perceive

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return mapM(observable, mutation_res)

    def perceive_action(self, impulse: PerceiveActionImpulse) -> Observable[PerceivedImpulse]:
        perceive_action_req: Callable[[PerceptionReq], None] = lambda x: x.perceive_action(impulse, lambda e: e.id())
        mutation_req: Callable[[MutationReq], None] = lambda x: x.perceive(perceive_action_req)
        mutation_res: Callable[[MutationRes], PerceivedImpulse] = lambda x: x.perceive.perceive_action

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return mapM(observable, mutation_res)

    def perceive_data(self, impulse: PerceiveDataImpulse) -> Observable[PerceivedImpulse]:
        perceive_data_req: Callable[[PerceptionReq], None] = lambda x: x.perceive_data(impulse, lambda e: e.id())
        mutation_req: Callable[[MutationReq], None] = lambda x: x.perceive(perceive_data_req)
        mutation_res: Callable[[MutationRes], PerceivedImpulse] = lambda x: x.perceive.perceive_data

        observable = rx.from_callable(lambda: self.client.mutation(GaiaRequest.mutation(mutation_req)),
                                      self._scheduler)
        return mapM(observable, mutation_res)



from typing import List


class IdentityImportImpulse:
    uri: str
    tenant_id: str
    identity_id: str
    identity_name: str
    override: bool

    def __init__(self, uri: str, tenant_id: str, identity_id: str, identity_name: str, override: bool):
        self.uri = uri
        self.tenant_id = tenant_id
        self.identity_id = identity_id
        self.identity_name = identity_name
        self.override = override

    def __eq__(self, other):
        return self.uri == other.uri and self.tenant_id == other.tenant_id and self.identity_id == other.identity_id \
               and self.identity_name == other.identity_name and self.override == other.override

    def __repr__(self):
        return {
            'uri': self.uri,
            'tenantId': self.tenant_id,
            'identityId': self.identity_id,
            'identityName': self.identity_name,
            'override': self.override
        }





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class PerceiveSubmitImpulse():
    """
    Input for submit perception impulse
    """
    name: str
    value: str

    def __init__(self, name: str, value: str):
        self.name = name
        self.value = value

    def __eq__(self, other):
        if type(other) is type(self):
            return self.name == other.name and self.value == other.value
        return False

    def __repr__(self):
        return {'name': self.name, 'value': self.value}




from gaia_sdk.graphql.response.type.Fulfilment import Fulfilment
from gaia_sdk.graphql.response.type.User import User
from gaia_sdk.graphql.response.type.ApiKey import ApiKey
from gaia_sdk.graphql.response.type.Behaviour import Behaviour
from gaia_sdk.graphql.response.type.Statement import Statement
from gaia_sdk.graphql.response.type.Intent import Intent
from gaia_sdk.graphql.response.type.Code import Code
from gaia_sdk.graphql.response.type.Role import Role
from gaia_sdk.graphql.response.type.SkillProvision import SkillProvision
from gaia_sdk.graphql.response.type.Skill import Skill
from gaia_sdk.graphql.response.type.Tenant import Tenant
from gaia_sdk.graphql.response.type.Prompt import Prompt
from gaia_sdk.graphql.response.type.Identity import Identity
from gaia_sdk.graphql.response.type.Edge import Edge

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Knowledge:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def users(self) -> List[User]:
        return list(map(lambda x: User(x), self.dictionary.get("users")))
    @property
    def user(self) -> User:
        return User(self.dictionary.get("user"))
    @property
    def api_keys(self) -> List[ApiKey]:
        return list(map(lambda x: ApiKey(x), self.dictionary.get("apiKeys")))
    @property
    def api_key(self) -> ApiKey:
        return ApiKey(self.dictionary.get("apiKey"))
    @property
    def roles(self) -> List[Role]:
        return list(map(lambda x: Role(x), self.dictionary.get("roles")))
    @property
    def role(self) -> Role:
        return Role(self.dictionary.get("role"))
    @property
    def tenants(self) -> List[Tenant]:
        return list(map(lambda x: Tenant(x), self.dictionary.get("tenants")))
    @property
    def tenant(self) -> Tenant:
        return Tenant(self.dictionary.get("tenant"))
    @property
    def identities(self) -> List[Identity]:
        return list(map(lambda x: Identity(x), self.dictionary.get("identities")))
    @property
    def identity(self) -> Identity:
        return Identity(self.dictionary.get("identity"))
    @property
    def intents(self) -> List[Intent]:
        return list(map(lambda x: Intent(x), self.dictionary.get("intents")))
    @property
    def intent(self) -> Intent:
        return Intent(self.dictionary.get("intent"))
    @property
    def prompts(self) -> List[Prompt]:
        return list(map(lambda x: Prompt(x), self.dictionary.get("prompts")))
    @property
    def prompt(self) -> Prompt:
        return Prompt(self.dictionary.get("prompt"))
    @property
    def fulfilments(self) -> List[Fulfilment]:
        return list(map(lambda x: Fulfilment(x), self.dictionary.get("fulfilments")))
    @property
    def fulfilment(self) -> Fulfilment:
        return Fulfilment(self.dictionary.get("fulfilment"))
    @property
    def statements(self) -> List[Statement]:
        return list(map(lambda x: Statement(x), self.dictionary.get("statements")))
    @property
    def statement(self) -> Statement:
        return Statement(self.dictionary.get("statement"))
    @property
    def codes(self) -> List[Code]:
        return list(map(lambda x: Code(x), self.dictionary.get("codes")))
    @property
    def code(self) -> Code:
        return Code(self.dictionary.get("code"))
    @property
    def behaviours(self) -> List[Behaviour]:
        return list(map(lambda x: Behaviour(x), self.dictionary.get("behaviours")))
    @property
    def behaviour(self) -> Behaviour:
        return Behaviour(self.dictionary.get("behaviour"))
    @property
    def edges(self) -> List[Edge]:
        return list(map(lambda x: Edge(x), self.dictionary.get("edges")))
    @property
    def edge(self) -> Edge:
        return Edge(self.dictionary.get("edge"))
    @property
    def skills(self) -> List[Skill]:
        return list(map(lambda x: Skill(x), self.dictionary.get("skills")))
    @property
    def skill(self) -> Skill:
        return Skill(self.dictionary.get("skill"))
    @property
    def skill_provisions(self) -> List[SkillProvision]:
        return list(map(lambda x: SkillProvision(x), self.dictionary.get("skillProvisions")))
    @property
    def skill_provision(self) -> SkillProvision:
        return SkillProvision(self.dictionary.get("skillProvision"))




from gaia_sdk.graphql.request.type.Role import Role

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class CreatedRoleImpulse(list):
    """
    Impulse which indicates the result of a create role impulse
    """

    def id(self):
        self.append(lambda x: "id")

    def data(self, config: Callable[['Role'], None]):
        def callback(registry: VariableRegistry):
            entity = Role()
            config(entity)
            return "data {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



import base64
import hashlib
import hmac


class HMAC:
    """
    This class is used to encrypt the data before sending it to G.A.I.A.
    """

    def __init__(self, secret):
        self.secret = secret

    def hash512(self, data):
        """
        Hashes the given string data.
        """
        key = self.secret.encode("UTF-8")
        msg = data.encode("UTF-8")

        dig = hmac.new(key, msg=msg, digestmod=hashlib.sha512).hexdigest()
        return base64.b64encode(dig.encode("utf-8")).decode()

    def hash(self, data):
        """
        Hashes the given string data.
        """
        key = self.secret.encode("UTF-8")
        msg = data.encode("UTF-8")

        dig = hmac.new(key, msg=msg, digestmod=hashlib.sha256).digest()
        return base64.b64encode(dig).decode()





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class CreateEdgeImpulse():
    """
    The specification to create an edge instance
    """
    source: str
    target: str
    type: str
    weight: float
    properties: dict

    def __init__(self, source: str, target: str, type: str, weight: float, properties: dict):
        self.source = source
        self.target = target
        self.type = type
        self.weight = weight
        self.properties = properties

    def __eq__(self, other):
        if type(other) is type(self):
            return self.source == other.source and self.target == other.target and self.type == other.type and self.weight == other.weight and self.properties == other.properties
        return False

    def __repr__(self):
        return {'source': self.source, 'target': self.target, 'type': self.type, 'weight': self.weight, 'properties': self.properties}




from gaia_sdk.graphql.request.type.StreamingImpulse import StreamingImpulse
from gaia_sdk.graphql.request.input.StreamImpulse import StreamImpulse

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Practice(list):
    """
    This type contains all practice sensor impulses which are used to support
    practice in gaia.
    """

    """
    Stream practice preparation impulse used to transfer a skill to gaia.
        This perception impulse do not invoke the data transmission but establishes
        a connection to the streaming api.
    """
    def prepare(self, impulse: StreamImpulse, config: Callable[['StreamingImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = StreamingImpulse()
            config(entity)
            return f'prepare(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.request.type.SkillIntrospection import SkillIntrospection

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Introspection(list):

    def cpu(self):
        self.append(lambda x: "cpu")

    def gpu(self):
        self.append(lambda x: "gpu")

    def memory(self):
        self.append(lambda x: "memory")

    def state(self):
        self.append(lambda x: "state")

    def started(self):
        self.append(lambda x: "started")

    def skills(self, config: Callable[['SkillIntrospection'], None]):
        def callback(registry: VariableRegistry):
            entity = SkillIntrospection()
            config(entity)
            return "skills {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class CreateFulfilmentImpulse():
    """
    The specification to create a fulfilment instance
    """
    identityId: str
    qualifier: str
    appendent: str
    utterance: dict
    labelList: List[str]

    def __init__(self, identityId: str, qualifier: str, appendent: str, utterance: dict, labelList: List[str]):
        self.identityId = identityId
        self.qualifier = qualifier
        self.appendent = appendent
        self.utterance = utterance
        self.labelList = labelList

    def __eq__(self, other):
        if type(other) is type(self):
            return self.identityId == other.identityId and self.qualifier == other.qualifier and self.appendent == other.appendent and self.utterance == other.utterance and self.labelList == other.labelList
        return False

    def __repr__(self):
        return {'identityId': self.identityId, 'qualifier': self.qualifier, 'appendent': self.appendent, 'utterance': self.utterance, 'labelList': self.labelList}





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Tenant(list):
    """
    Represents tenant information
    """

    """
    The tenant id
    """
    def tenant_id(self):
        self.append(lambda x: "tenantId")

    """
    The name of the tenant
    """
    def qualifier(self):
        self.append(lambda x: "qualifier")

    """
    The list of implicit identities
    """
    def implicit_identities(self):
        self.append(lambda x: "implicitIdentities")

    """
    The list of explicit identities
    """
    def explicit_identities(self):
        self.append(lambda x: "explicitIdentities")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



import struct


class ByteBuffer():

    def __init__(self, buffer: bytearray = bytearray()):
        self.buffer = buffer

    def put(self, obj: bytes):
        self.buffer = self.buffer + obj

    def put_long(self, obj: int):
        self.buffer = self.buffer + struct.pack('>q', obj)

    def has_remaining(self):
        return len(self.buffer) > 0

    def get_long(self):
        bytes = self.buffer[:8]
        self.buffer = self.buffer[8:]
        return struct.unpack('>q', bytes)[0]

    def to_bytes(self):
        return self.buffer




from gaia_sdk.graphql.response.type.ConnectNodeRemovedImpulse import ConnectNodeRemovedImpulse
from gaia_sdk.graphql.response.type.ConnectNodeUnsetImpulse import ConnectNodeUnsetImpulse
from gaia_sdk.graphql.response.type.ConnectNodeAppendedImpulse import ConnectNodeAppendedImpulse
from gaia_sdk.graphql.response.type.ConnectNodeSetImpulse import ConnectNodeSetImpulse
from gaia_sdk.graphql.request.input.ConnectSetNodeImpulse import ConnectSetNodeImpulse
from gaia_sdk.graphql.request.input.ConnectAppendNodeImpulse import ConnectAppendNodeImpulse
from gaia_sdk.graphql.request.input.ConnectUnsetNodeImpulse import ConnectUnsetNodeImpulse
from gaia_sdk.graphql.request.input.ConnectRemoveNodeImpulse import ConnectRemoveNodeImpulse

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class ConnectNodeKnowledge:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def append(self) -> ConnectNodeAppendedImpulse:
        return ConnectNodeAppendedImpulse(self.dictionary.get("append"))
    @property
    def remove(self) -> ConnectNodeRemovedImpulse:
        return ConnectNodeRemovedImpulse(self.dictionary.get("remove"))
    @property
    def set(self) -> ConnectNodeSetImpulse:
        return ConnectNodeSetImpulse(self.dictionary.get("set"))
    @property
    def unset(self) -> ConnectNodeUnsetImpulse:
        return ConnectNodeUnsetImpulse(self.dictionary.get("unset"))





from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Role:
    """
    Represents Role information
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    Id of the tenant
    """
    @property
    def tenant_id(self) -> Uuid:
        return Uuid(self.dictionary.get("tenantId"))
    """
    Id of the role
    """
    @property
    def role_id(self) -> Uuid:
        return Uuid(self.dictionary.get("roleId"))
    """
    The name of the role
    """
    @property
    def name(self) -> String:
        return String(self.dictionary.get("name"))
    """
    The permissions of the role
    """
    @property
    def permissions(self) -> List[String]:
        return list(map(lambda x: String(x), self.dictionary.get("permissions")))




from gaia_sdk.graphql.request.type.DeletedApiKeyImpulse import DeletedApiKeyImpulse
from gaia_sdk.graphql.request.type.DeletedFulfilmentImpulse import DeletedFulfilmentImpulse
from gaia_sdk.graphql.request.type.DeletedBehaviourImpulse import DeletedBehaviourImpulse
from gaia_sdk.graphql.request.type.DeletedTenantImpulse import DeletedTenantImpulse
from gaia_sdk.graphql.request.type.DeletedSkillProvisionImpulse import DeletedSkillProvisionImpulse
from gaia_sdk.graphql.request.type.DeletedStatementImpulse import DeletedStatementImpulse
from gaia_sdk.graphql.request.type.DeletedSkillImpulse import DeletedSkillImpulse
from gaia_sdk.graphql.request.type.DeletedRoleImpulse import DeletedRoleImpulse
from gaia_sdk.graphql.request.type.DeletedCodeImpulse import DeletedCodeImpulse
from gaia_sdk.graphql.request.type.DeletedEdgeImpulse import DeletedEdgeImpulse
from gaia_sdk.graphql.request.type.DeletedIdentityImpulse import DeletedIdentityImpulse
from gaia_sdk.graphql.request.type.DeletedIntentImpulse import DeletedIntentImpulse
from gaia_sdk.graphql.request.type.DeletedPromptImpulse import DeletedPromptImpulse
from gaia_sdk.graphql.request.type.DeletedUserImpulse import DeletedUserImpulse
from gaia_sdk.graphql.request.input.DeleteFulfilmentImpulse import DeleteFulfilmentImpulse
from gaia_sdk.graphql.request.input.DeleteCodeImpulse import DeleteCodeImpulse
from gaia_sdk.graphql.request.input.DeleteEdgeImpulse import DeleteEdgeImpulse
from gaia_sdk.graphql.request.input.DeleteBehaviourImpulse import DeleteBehaviourImpulse
from gaia_sdk.graphql.request.input.DeleteSkillImpulse import DeleteSkillImpulse
from gaia_sdk.graphql.request.input.DeleteIdentityImpulse import DeleteIdentityImpulse
from gaia_sdk.graphql.request.input.DeleteUserImpulse import DeleteUserImpulse
from gaia_sdk.graphql.request.input.DeleteApiKeyImpulse import DeleteApiKeyImpulse
from gaia_sdk.graphql.request.input.DeleteStatementImpulse import DeleteStatementImpulse
from gaia_sdk.graphql.request.input.DeletePromptImpulse import DeletePromptImpulse
from gaia_sdk.graphql.request.input.DeleteTenantImpulse import DeleteTenantImpulse
from gaia_sdk.graphql.request.input.DeleteIntentImpulse import DeleteIntentImpulse
from gaia_sdk.graphql.request.input.DeleteRoleImpulse import DeleteRoleImpulse
from gaia_sdk.graphql.request.input.DeleteSkillProvisionImpulse import DeleteSkillProvisionImpulse

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class DeleteKnowledge(list):

    """
    deletes a list of identities with the given specifications
    """
    def identities(self, impulses: List[DeleteIdentityImpulse], config: Callable[['DeletedIdentityImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedIdentityImpulse()
            config(entity)
            return f'identities(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of tenants with the given specifications
    """
    def tenants(self, impulses: List[DeleteTenantImpulse], config: Callable[['DeletedTenantImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedTenantImpulse()
            config(entity)
            return f'tenants(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of users with the given specifications
    """
    def users(self, impulses: List[DeleteUserImpulse], config: Callable[['DeletedUserImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedUserImpulse()
            config(entity)
            return f'users(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of api keys with the given specifications
    """
    def api_keys(self, impulses: List[DeleteApiKeyImpulse], config: Callable[['DeletedApiKeyImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedApiKeyImpulse()
            config(entity)
            return f'apiKeys(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of roles with the given specifications
    """
    def roles(self, impulses: List[DeleteRoleImpulse], config: Callable[['DeletedRoleImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedRoleImpulse()
            config(entity)
            return f'roles(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of intents with the given specifications
    """
    def intents(self, impulses: List[DeleteIntentImpulse], config: Callable[['DeletedIntentImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedIntentImpulse()
            config(entity)
            return f'intents(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of prompts with the given specifications
    """
    def prompts(self, impulses: List[DeletePromptImpulse], config: Callable[['DeletedPromptImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedPromptImpulse()
            config(entity)
            return f'prompts(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of statements with the given specifications
    """
    def statements(self, impulses: List[DeleteStatementImpulse], config: Callable[['DeletedStatementImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedStatementImpulse()
            config(entity)
            return f'statements(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of fulfilments with the given specifications
    """
    def fulfilments(self, impulses: List[DeleteFulfilmentImpulse], config: Callable[['DeletedFulfilmentImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedFulfilmentImpulse()
            config(entity)
            return f'fulfilments(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of behaviours with the given specifications
    """
    def behaviours(self, impulses: List[DeleteBehaviourImpulse], config: Callable[['DeletedBehaviourImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedBehaviourImpulse()
            config(entity)
            return f'behaviours(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of codes with the given specifications
    """
    def codes(self, impulses: List[DeleteCodeImpulse], config: Callable[['DeletedCodeImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedCodeImpulse()
            config(entity)
            return f'codes(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of edges with the given specifications
    """
    def edges(self, impulses: List[DeleteEdgeImpulse], config: Callable[['DeletedEdgeImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedEdgeImpulse()
            config(entity)
            return f'edges(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of skills with the given specifications
    """
    def skills(self, impulses: List[DeleteSkillImpulse], config: Callable[['DeletedSkillImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedSkillImpulse()
            config(entity)
            return f'skills(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    deletes a list of skill provisions with the given specifications
    """
    def skill_provisions(self, impulses: List[DeleteSkillProvisionImpulse], config: Callable[['DeletedSkillProvisionImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = DeletedSkillProvisionImpulse()
            config(entity)
            return f'skillProvisions(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Role(list):
    """
    Represents Role information
    """

    """
    Id of the tenant
    """
    def tenant_id(self):
        self.append(lambda x: "tenantId")

    """
    Id of the role
    """
    def role_id(self):
        self.append(lambda x: "roleId")

    """
    The name of the role
    """
    def name(self):
        self.append(lambda x: "name")

    """
    The permissions of the role
    """
    def permissions(self):
        self.append(lambda x: "permissions")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.response.type.Conversational import Conversational
from gaia_sdk.graphql.response.type.PerceivedImpulse import PerceivedImpulse
from gaia_sdk.graphql.request.input.PerceiveDataImpulse import PerceiveDataImpulse
from gaia_sdk.graphql.request.input.PerceiveActionImpulse import PerceiveActionImpulse

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Perception:
    """
    This type contains all perception sensor impulses which are used to invoke
    events in gaia.
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    Contains all perception fields needed for a conversation.
    """
    @property
    def conversational(self) -> Conversational:
        return Conversational(self.dictionary.get("conversational"))
    """
    Data perception impulse used to invoke a data transformation behaviour
    """
    @property
    def perceive_data(self) -> PerceivedImpulse:
        return PerceivedImpulse(self.dictionary.get("perceiveData"))
    """
    Action perception impulse used to invoke a data transformation behaviour
    """
    @property
    def perceive_action(self) -> PerceivedImpulse:
        return PerceivedImpulse(self.dictionary.get("perceiveAction"))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Prompt(list):
    """
    Represents prompt information
    """

    """
    The prompt id
    """
    def identity_id(self):
        self.append(lambda x: "identityId")

    """
    The prompt reference id
    """
    def reference(self):
        self.append(lambda x: "reference")

    """
    The name of the prompt
    """
    def qualifier(self):
        self.append(lambda x: "qualifier")

    """
    Detailed description about the prompt
    """
    def appendent(self):
        self.append(lambda x: "appendent")

    """
    The utterance dictionary. The key is a language key and the value is a list of utterances
    """
    def utterance(self):
        self.append(lambda x: "utterance")

    """
    The list of labels of the prompt
    """
    def label_list(self):
        self.append(lambda x: "labelList")

    """
    The version of the prompt
    """
    def version(self):
        self.append(lambda x: "version")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



from requests import Response
from typing import Dict
from typing import Any

from gaia_sdk.api.client_options import ClientOptions
from gaia_sdk.api.transporter.abstract_transporter import ITransporter
from gaia_sdk.http.request.Payload import Payload
from copy import deepcopy


class GaiaStreamClient(object):
    transporter: ITransporter
    options: ClientOptions

    def __init__(self, transporter: ITransporter, options: ClientOptions):
        self.transporter = transporter
        self.options = options

    def __eq__(self, other):
        return self.options == other.options

    def __repr__(self):
        return {'options': self.options}

    def post_json(self, payload, url_postfix: str = "") -> Response:
        if self._is_dataclass(payload) and not isinstance(payload, type):
            payload = payload.__repr__()
        return self.transporter.transport(self.options, Payload.json(payload), url_postfix)

    def post_form_data(self, payload, url_postfix: str = "") -> Response:
        return self.transporter.transport(self.options, Payload.form_data(payload), url_postfix)

    def post_stream(self, data, request_parameters: Dict[str, Any], url_postfix: str = "") -> Response:
        opts = self.options\
            .with_request_parameters(request_parameters)\
            .with_content_type("application/octet-stream")
        return self.transporter.transport(opts, Payload.stream(data), url_postfix)

    @staticmethod
    def _is_dataclass(obj) -> bool:
        return isinstance(obj.__repr__(), dict)



from gaia_sdk.graphql.GaiaClientFactory import GaiaClientFactory
from gaia_sdk.graphql.GaiaClientBuilder import GaiaHttpClientBuilder
from gaia_sdk.graphql.GaiaClient import GaiaClient
from gaia_sdk.api.GaiaCredentials import GaiaCredentials

from gaia_sdk.api.client_options import ClientOptions
from gaia_sdk.http.request.Payload import Payload
from gaia_sdk.http.GaiaStreamClient import GaiaStreamClient
from gaia_sdk.http.GaiaStreamClientBuilder import GaiaStreamClientFactory
from gaia_sdk.http.GaiaStreamClientBuilder import GaiaHttpStreamClientBuilder
from requests import Response
from gaia_sdk.api.transporter.abstract_transporter import ITransporter

from gaia_sdk.api.GaiaCredentials import HMACCredentials
from gaia_sdk.gaia import Gaia
import json

def mock_gaia_ref(mock_response_handler):
    mock_client_factory = MockClientFactory()
    mock_client_factory.set_mock_response_handler(mock_response_handler)

    mock_stream_client_factory = MocktreamClientFactory()
    mock_stream_client_factory.set_mock_response_handler(mock_response_handler)

    Gaia._client_factory = mock_client_factory
    Gaia._stream_client_factory = mock_stream_client_factory

    return Gaia.connect("http://localhost:8080", HMACCredentials("mockedApiKey", "mockedApiSecret"))

class MockResponse(object):

    def __init__(self, content):
        self.content = content

    def json(self):
        return self.content

    def data(self):
        return self.content

class MockRequest(object):

    def __init__(self, options: ClientOptions, payload: Payload, url_post_fix: str = ""):
        self.options = options
        self.payload = payload
        self.url_post_fix = url_post_fix

    def __str__(self):
        return json.dumps({"options": { "content_type": self.options.content_type, "request_parameters": self.options.request_parameters}, "payload": self.payload.payload_type, "url_post_fix": self.url_post_fix})

class MockClientFactory(GaiaClientFactory):

    def http(self, url):
        self.client_builder = MockHttpClientBuilder(url)
        self.client_builder.set_mock_response_handler(self.mock_response_handler)
        return self.client_builder

    def set_mock_response_handler(self, mock_response_handler):
        self.mock_response_handler = mock_response_handler
        if hasattr(self, 'client_builder'):
            self.client_builder.set_mock_response_handler(mock_response_handler)

    def set_mock_response(self, mock_response):
        self.set_mock_response_handler(lambda request: mock_response)

class MocktreamClientFactory(GaiaStreamClientFactory):

    def http(self, url):
        self.client_builder = MockHttpStreamClientBuilder(url)
        self.client_builder.set_mock_response_handler(self.mock_response_handler)
        return self.client_builder

    def set_mock_response_handler(self, mock_response_handler):
        self.mock_response_handler = mock_response_handler
        if hasattr(self, 'client_builder'):
            self.client_builder.mock_response_handler(mock_response_handler)

    def set_mock_response(self, mock_response):
        self.set_mock_response_handler(lambda request: mock_response)

class MockTransporter(ITransporter):

    def set_mock_response_handler(self, mock_response_handler):
        self.mock_response_handler = mock_response_handler

    def transport(self, options: ClientOptions, payload: Payload, url_post_fix: str = "") -> Response:
        return self.mock_response_handler(MockRequest(options, payload, url_post_fix))

class MockHttpClientBuilder(GaiaHttpClientBuilder):

    def __init__(self, url):
        self.url = url

    def with_credentials(self, credentials):
        self.credentials = credentials
        return self

    def build(self):
        self.transporter = MockTransporter()
        self.set_mock_response_handler(self.mock_response_handler)
        return GaiaClient(self.transporter, ClientOptions(self.credentials))

    def set_mock_response_handler(self, mock_response_handler):
        self.mock_response_handler = mock_response_handler
        if hasattr(self, 'transporter'):
            self.transporter.set_mock_response_handler(mock_response_handler)

class MockHttpStreamClientBuilder(GaiaHttpStreamClientBuilder):

    def __init__(self, url):
        self.url = url

    def with_credentials(self, credentials: GaiaCredentials):
        self.credentials = credentials
        return self

    def build(self):
        self.transporter = MockTransporter()
        self.set_mock_response_handler(self.mock_response_handler)
        return GaiaStreamClient(self.transporter, ClientOptions(self.credentials))

    def set_mock_response_handler(self, mock_response_handler):
        self.mock_response_handler = mock_response_handler
        if hasattr(self, 'transporter'):
            self.transporter.set_mock_response_handler(mock_response_handler)





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class CreateRoleImpulse():
    """
    The specification to create a role instance
    """
    tenantId: str
    name: str
    permissions: List[str]

    def __init__(self, tenantId: str, name: str, permissions: List[str]):
        self.tenantId = tenantId
        self.name = name
        self.permissions = permissions

    def __eq__(self, other):
        if type(other) is type(self):
            return self.tenantId == other.tenantId and self.name == other.name and self.permissions == other.permissions
        return False

    def __repr__(self):
        return {'tenantId': self.tenantId, 'name': self.name, 'permissions': self.permissions}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gaia_sdk',
 'gaia_sdk.api',
 'gaia_sdk.api.tests',
 'gaia_sdk.api.transporter',
 'gaia_sdk.graphql',
 'gaia_sdk.tests']

package_data = \
{'': ['*'],
 'gaia_sdk': ['http/*', 'http/request/*', 'http/response/*', 'http/tests/*'],
 'gaia_sdk.api': ['rx/*'],
 'gaia_sdk.graphql': ['request/enumeration/*',
                      'request/input/*',
                      'request/intf/*',
                      'request/type/*',
                      'response/intf/*',
                      'response/type/*']}

install_requires = \
['Rx>=3.1.1,<4.0.0', 'requests>=2.23.0,<3.0.0']

entry_points = \
{'console_scripts': ['build = poetry_scripts:build',
                     'clean = poetry_scripts:clean',
                     'install = poetry_scripts:install',
                     'publish = poetry_scripts:publish',
                     'test = poetry_scripts:test']}

setup_kwargs = {
    'name': 'gaia-sdk',
    'version': '3.3.0rc4',
    'description': 'Python SDK for the GAIA ecosystem.',
    'long_description': None,
    'author': 'Leftshift One',
    'author_email': 'contact@leftshift.one',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from gaia_sdk.api.client_options import ClientOptions
from gaia_sdk.http.HttpTransporter import HttpTransporter
from gaia_sdk.graphql.GaiaClient import GaiaClient

#Class generated from template src/main/resources/template/python/ClientBuilderTemplate.vm

class GaiaHttpClientBuilder(object):

    def __init__(self, url):
        self.url = url

    def with_credentials(self, credentials):
        self.credentials = credentials
        return self

    def build(self):
        return GaiaClient(HttpTransporter(self.url), ClientOptions(self.credentials))



from enum import Enum


class EdgeType(Enum):
    TenantIdentity = 1

    Supervisor = 2

    IdentityNerSkill = 3

    IdentityErrorStatement = 4

    IdentityUnknownBehaviour = 5

    IdentityDiscontinuableStatement = 6

    IdentityWelcomeBehaviour = 7

    IdentityIntentDetectionSkill = 8

    UserRole = 9

    ApiKeyRole = 10

    IdentityDisambiguationBehaviour = 11






from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class SkillProvision(list):
    """
    Represents SkillProvision information
    """

    """
    Id of the tenant
    """
    def tenant_id(self):
        self.append(lambda x: "tenantId")

    """
    Skill reference
    """
    def reference(self):
        self.append(lambda x: "reference")

    """
    The name of the skill provision
    """
    def qualifier(self):
        self.append(lambda x: "qualifier")

    """
    Detailed description about the skill provision
    """
    def appendent(self):
        self.append(lambda x: "appendent")

    """
    The list of labels of the skill provision
    """
    def label_list(self):
        self.append(lambda x: "labelList")

    """
    The version of the skill
    """
    def version(self):
        self.append(lambda x: "version")

    """
    The reference to skill
    """
    def skill_ref(self):
        self.append(lambda x: "skillRef")

    """
    CPU
    """
    def cpu(self):
        self.append(lambda x: "cpu")

    """
    Memory
    """
    def memory(self):
        self.append(lambda x: "memory")

    """
    Amount of replicas
    """
    def replicas(self):
        self.append(lambda x: "replicas")

    """
    Whether the skill provision should be enabled or not
    """
    def enabled(self):
        self.append(lambda x: "enabled")

    """
    Amount of seconds that the skill requires to be ready
    """
    def bootstrap_timeout(self):
        self.append(lambda x: "bootstrapTimeout")

    """
    Value-Key pairs with information needed for the skill provision
    """
    def environment(self):
        self.append(lambda x: "environment")

    """
    Whether the skill provision has been built
    """
    def built(self):
        self.append(lambda x: "built")

    """
    The current status of the skill provision
    """
    def status(self):
        self.append(lambda x: "status")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



import unittest

import rx.operators as ops

from gaia_sdk.tests.mock import MockResponse
from gaia_sdk.tests.mock import mock_gaia_ref


class TestSkill(unittest.TestCase):

    def test_sends_and_retrieves(self):
        def mock(request):
            self.assertEqual(request.url_post_fix, "/skill/evaluate")
            return MockResponse(b'{"response":"hello back",":namespace":"outgoing"}')

        gaia_ref = mock_gaia_ref(mock)
        skill_ref = gaia_ref.skill("skillProvision://8db77283-f25b-4cbb-8d26-692bb2672fb3/test")
        result = skill_ref.evaluate(payload={"request": "hello world!"}).run()
        self.assertEqual(result.response, {'response': 'hello back', ':namespace': 'outgoing'})

    def test_cancel(self):
        def mock(request):
            self.assertEqual(request.url_post_fix, "/skill/cancel")
            return MockResponse(b'{"reference": "bf8929ab-b432-470e-acb2-bce380f61333"}')

        gaia_ref = mock_gaia_ref(mock)
        skill_ref = gaia_ref.skill("skillBuild://bf8929ab-b432-470e-acb2-bce380f61333/bla")
        result = skill_ref.cancel().run()
        self.assertEqual(result.reference, "bf8929ab-b432-470e-acb2-bce380f61333")

    def test_logs(self):
        def mock(request):
            self.assertEqual(request.url_post_fix, "/skill/logs")
            return MockResponse(b'{"logLines": ["a", "b", "c"]}')
        gaia_ref = mock_gaia_ref(mock)
        skill_ref = gaia_ref.skill("skillBuild://8db77283-f25b-4cbb-8d26-692bb2672fb3/first-7ac19-build")
        r = skill_ref.logs(30) \
            .pipe(ops.to_list()) \
            .run()
        self.assertEqual(r, ['a', 'b', 'c'])

    def test_build(self):
        def mock(request):
            self.assertEqual(request.url_post_fix, "/skill/build")
            return MockResponse(b'{"reference": "1234"}')

        gaia_ref = mock_gaia_ref(mock)
        skill_ref = gaia_ref.skill("skillProvision://8db77283-f25b-4cbb-8d26-692bb2672fb3/some-prov-1")
        r = skill_ref.build().run()
        self.assertEqual(r.reference, "1234")


from uuid import UUID
Uuid = str
class VariableRegistry:

    def __init__(self):
        self.variables = dict()
        self.datatypes = []
        self.counters = dict()

    def register(self, name: str, value):
        if value == None:
            return "null"

        if name not in self.counters:
            self.counters[name] = 0
        self.counters[name] = self.counters[name] + 1
        varName = name + str(self.counters[name])

        self.variables[varName] = self.to_value(value)
        self.datatypes.append("$" + varName + ":" + self.toType(value))

        return "$" + varName

    def getVariables(self):
        return self.variables

    def getDatatypes(self):
        return self.datatypes

    def toType(self, obj) -> str:
        if isinstance(obj, list):
            return "[" + str(self.toType(obj[0])) + "]"
        if (type(obj) is str):
            return self.resolveString(obj)
        if (type(obj) is int):
            return "Int"
        if (type(obj) is float):
            return "float"
        if (type(obj) is bool):
            return "Boolean"
        else:
            return type(obj).__name__

    def resolveString(self, obj):
        if (self.isUUID(obj)):
            return "Uuid"
        else:
            return "String"

    def isUUID(self, obj):
        try:
            UUID(obj, version=4)
            return True
        except ValueError:
            return False

    def to_value(self, obj):
        if isinstance(obj, list):
            return list(map(lambda x: self.to_value(x), obj))

        if (type(obj) is str):
            return obj
        if (type(obj) is int):
            return obj
        if (type(obj) is float):
            return obj
        if (type(obj) is bool):
            return obj
        else:
            return obj.__dict__
